@@ -1,5 +1,6 @@
 import importlib
 import os
+import warnings
 from functools import partial
 
 from qtpy.QtCore import QSize, Qt, QThread
@@ -7,9 +8,11 @@ from qtpy.QtGui import QIcon
 from qtpy.QtWidgets import QGridLayout, QMainWindow, QMessageBox, QProgressBar, QToolButton, QWidget
 
 from PartSeg import ANALYSIS_NAME, APP_NAME, MASK_NAME
-from PartSeg.common_backend.base_settings import BaseSettings
+from PartSeg.common_backend import napari_get_settings
+from PartSeg.common_backend.base_settings import BaseSettings, get_stylesheet, get_theme, napari_template
 from PartSeg.common_backend.load_backup import import_config
 from PartSeg.common_gui.main_window import BaseMainWindow
+from PartSegCore import state_store
 from PartSegData import icons_dir
 from PartSegImage import TiffImageReader
 
@@ -69,6 +72,15 @@ class MainWindow(QMainWindow):
         self.setWindowIcon(analysis_icon)
         self.prepare = None
         self.wind = None
+        self._update_theme()
+
+    def _update_theme(self):
+        napari_settings = napari_get_settings(state_store.save_folder)
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", FutureWarning)
+            theme = get_theme(napari_settings.appearance.theme)
+        # TODO understand qss overwrite mechanism
+        self.setStyleSheet(napari_template(get_stylesheet(), **theme))
 
     def _launch_begin(self):
         self.progress.setVisible(True)
