@@ -29,6 +29,7 @@ from qtpy.QtWidgets import (
     QVBoxLayout,
     QWidget,
 )
+from superqt import QEnumComboBox
 
 from PartSeg.common_gui.advanced_tabs import AdvancedWindow
 from PartSegCore.analysis.algorithm_description import analysis_algorithm_dict
@@ -40,7 +41,6 @@ from PartSegData import icons_dir
 from ..common_gui.custom_save_dialog import FormDialog
 from ..common_gui.lock_checkbox import LockCheckBox
 from ..common_gui.searchable_list_widget import SearchableListWidget
-from ..common_gui.universal_gui_part import EnumComboBox
 from .measurement_widget import MeasurementWidget
 from .partseg_settings import PartSettings
 from .profile_export import ExportDialog, ImportDialog, ProfileDictViewer, StringViewer
@@ -94,8 +94,8 @@ class Properties(QWidget):
             el.setRange(0, 1000000)
             # noinspection PyUnresolvedReferences
             el.valueChanged.connect(self.image_spacing_change)
-        self.units = EnumComboBox(Units)
-        self.units.set_value(units_value)
+        self.units = QEnumComboBox(enum_class=Units)
+        self.units.setCurrentEnum(units_value)
         # noinspection PyUnresolvedReferences
         self.units.currentIndexChanged.connect(self.update_spacing)
 
@@ -143,12 +143,12 @@ class Properties(QWidget):
             return
         try:
             if self.sender() == self.profile_list.list_widget:
-                profile = self._settings.segmentation_profiles[text]
+                profile = self._settings.roi_profiles[text]
                 self.pipeline_list.selectionModel().clear()
                 self.delete_btn.setText("Delete profile")
                 self.rename_btn.setText("Rename profile")
             elif self.sender() == self.pipeline_list.list_widget:
-                profile = self._settings.segmentation_pipelines[text]
+                profile = self._settings.roi_pipelines[text]
                 self.profile_list.selectionModel().clear()
                 self.delete_btn.setText("Delete pipeline")
                 self.rename_btn.setText("Rename pipeline")
@@ -176,12 +176,12 @@ class Properties(QWidget):
         for el in self._settings.image_spacing:
             voxel_size *= el * UNIT_SCALE[self.units.currentIndex()]
         self.voxel_size_label.setText(
-            f"Voxel_size: {voxel_size} {self.units.get_value().name}" f"<sup>{len(self._settings.image_spacing)}</sup>"
+            f"Voxel_size: {voxel_size} {self.units.currentEnum().name} <sup>{len(self._settings.image_spacing)}</sup>"
         )
 
     def update_spacing(self, index=None):
         voxel_size = 1
-        value = self.units.get_value()
+        value = self.units.currentEnum()
         if index is not None:
             self._settings.set("units_value", value)
         for el, sp in zip(self.spacing[::-1], self._settings.image_spacing[::-1]):
@@ -192,15 +192,15 @@ class Properties(QWidget):
             el.blockSignals(False)
         self.spacing[0].setDisabled(len(self._settings.image_spacing) == 2)
         self.voxel_size_label.setText(
-            f"Voxel_size: {voxel_size} {value.name}" f"<sup>{len(self._settings.image_spacing)}</sup>"
+            f"Voxel_size: {voxel_size} {value.name} <sup>{len(self._settings.image_spacing)}</sup>"
         )
 
     def update_profile_list(self):
-        current_names = set(self._settings.segmentation_profiles.keys())
+        current_names = set(self._settings.roi_profiles.keys())
         self.profile_list.clear()
         self.profile_list.addItems(sorted(current_names))
         self.pipeline_list.clear()
-        self.pipeline_list.addItems(sorted(set(self._settings.segmentation_pipelines.keys())))
+        self.pipeline_list.addItems(sorted(set(self._settings.roi_pipelines.keys())))
         self.delete_btn.setDisabled(True)
         self.rename_btn.setDisabled(True)
         self.info_label.setPlainText("")
@@ -220,17 +220,17 @@ class Properties(QWidget):
         text, dkt = "", {}
         if self.profile_list.selectedItems():
             text = self.profile_list.selectedItems()[0].text()
-            dkt = self._settings.segmentation_profiles
+            dkt = self._settings.roi_profiles
         elif self.pipeline_list.selectedItems():
             text = self.pipeline_list.selectedItems()[0].text()
-            dkt = self._settings.segmentation_pipelines
+            dkt = self._settings.roi_pipelines
         if text != "":
             self.delete_btn.setDisabled(True)
             del dkt[text]
             self.update_profile_list()
 
     def export_profile(self):
-        exp = ExportDialog(self._settings.segmentation_profiles, ProfileDictViewer)
+        exp = ExportDialog(self._settings.roi_profiles, ProfileDictViewer)
         if not exp.exec_():
             return
         dial = QFileDialog(self, "Export profile segment")
@@ -245,7 +245,7 @@ class Properties(QWidget):
             file_path = dial.selectedFiles()[0]
             self._settings.set("io.save_directory", os.path.dirname(file_path))
             self._settings.add_path_history(os.path.dirname(file_path))
-            data = {x: self._settings.segmentation_profiles[x] for x in exp.get_export_list()}
+            data = {x: self._settings.roi_profiles[x] for x in exp.get_export_list()}
             with open(file_path, "w") as ff:
                 json.dump(data, ff, cls=self._settings.json_encoder_class, indent=2)
 
@@ -264,7 +264,7 @@ class Properties(QWidget):
             profs, err = self._settings.load_part(file_path)
             if err:
                 QMessageBox.warning(self, "Import error", "error during importing, part of data were filtered.")
-            profiles_dict = self._settings.segmentation_profiles
+            profiles_dict = self._settings.roi_profiles
             imp = ImportDialog(profs, profiles_dict, ProfileDictViewer)
             if not imp.exec_():
                 return
@@ -274,7 +274,7 @@ class Properties(QWidget):
             self.update_profile_list()
 
     def export_pipeline(self):
-        exp = ExportDialog(self._settings.segmentation_pipelines, ProfileDictViewer)
+        exp = ExportDialog(self._settings.roi_pipelines, ProfileDictViewer)
         if not exp.exec_():
             return
         dial = QFileDialog(self, "Export pipeline segment")
@@ -287,7 +287,7 @@ class Properties(QWidget):
         dial.setHistory(dial.history() + self._settings.get_path_history())
         if dial.exec_():
             file_path = dial.selectedFiles()[0]
-            data = {x: self._settings.segmentation_pipelines[x] for x in exp.get_export_list()}
+            data = {x: self._settings.roi_pipelines[x] for x in exp.get_export_list()}
             with open(file_path, "w") as ff:
                 json.dump(data, ff, cls=self._settings.json_encoder_class, indent=2)
             self._settings.set("io.save_directory", os.path.dirname(file_path))
@@ -307,7 +307,7 @@ class Properties(QWidget):
             profs, err = self._settings.load_part(file_path)
             if err:
                 QMessageBox.warning(self, "Import error", "error during importing, part of data were filtered.")
-            profiles_dict = self._settings.segmentation_pipelines
+            profiles_dict = self._settings.roi_pipelines
             imp = ImportDialog(profs, profiles_dict, ProfileDictViewer)
             if not imp.exec_():
                 return
@@ -320,10 +320,10 @@ class Properties(QWidget):
         profile_name, profiles_dict = "", {}
         if self.profile_list.selectedItems():
             profile_name = self.profile_list.selectedItems()[0].text()
-            profiles_dict = self._settings.segmentation_profiles
+            profiles_dict = self._settings.roi_profiles
         elif self.pipeline_list.selectedItems():
             profile_name = self.pipeline_list.selectedItems()[0].text()
-            profiles_dict = self._settings.segmentation_pipelines
+            profiles_dict = self._settings.roi_pipelines
         if profile_name == "":
             return
         text, ok = QInputDialog.getText(self, "New profile name", f"New name for {profile_name}", text=profile_name)
@@ -366,8 +366,8 @@ class MeasurementSettings(QWidget):
         self.profile_description.setReadOnly(True)
         self.profile_options = QListWidget()
         self.profile_options_chosen = QListWidget()
-        self.measurement_area_choose = EnumComboBox(AreaType)
-        self.per_component = EnumComboBox(PerComponent)
+        self.measurement_area_choose = QEnumComboBox(enum_class=AreaType)
+        self.per_component = QEnumComboBox(enum_class=PerComponent)
         self.power_num = QDoubleSpinBox()
         self.power_num.setDecimals(3)
         self.power_num.setRange(-100, 100)
@@ -529,8 +529,8 @@ class MeasurementSettings(QWidget):
             item = self.profile_options.currentItem()
             self.chosen_element_area = self.get_parameters(
                 deepcopy(item.stat),
-                self.measurement_area_choose.get_value(),
-                self.per_component.get_value(),
+                self.measurement_area_choose.currentEnum(),
+                self.per_component.currentEnum(),
                 self.power_num.value(),
             )
             if self.chosen_element_area is None:
@@ -539,8 +539,8 @@ class MeasurementSettings(QWidget):
             item.setIcon(QIcon(os.path.join(icons_dir, "task-accepted.png")))
         elif (
             self.profile_options.currentItem() == self.chosen_element
-            and self.measurement_area_choose.get_value() == self.chosen_element_area.area
-            and self.per_component.get_value() == self.chosen_element_area.per_component
+            and self.measurement_area_choose.currentEnum() == self.chosen_element_area.area
+            and self.per_component.currentEnum() == self.chosen_element_area.per_component
         ):
             self.chosen_element.setIcon(QIcon())
             self.chosen_element = None
@@ -548,8 +548,8 @@ class MeasurementSettings(QWidget):
             item: MeasurementListWidgetItem = self.profile_options.currentItem()
             leaf = self.get_parameters(
                 deepcopy(item.stat),
-                self.measurement_area_choose.get_value(),
-                self.per_component.get_value(),
+                self.measurement_area_choose.currentEnum(),
+                self.per_component.currentEnum(),
                 self.power_num.value(),
             )
             if leaf is None:
@@ -615,8 +615,7 @@ class MeasurementSettings(QWidget):
             self.save_butt.setDisabled(True)
             self.save_butt_with_name.setDisabled(True)
 
-    @staticmethod
-    def get_parameters(node: Union[Node, Leaf], area: AreaType, component: PerComponent, power: float):
+    def get_parameters(self, node: Union[Node, Leaf], area: AreaType, component: PerComponent, power: float):
         if isinstance(node, Node):
             return node
         node = node.replace_(power=power)
@@ -627,7 +626,7 @@ class MeasurementSettings(QWidget):
         try:
             arguments = MEASUREMENT_DICT[str(node.name)].get_fields()
             if len(arguments) > 0 and len(node.dict) == 0:
-                dial = FormDialog(arguments)
+                dial = FormDialog(arguments, settings=self.settings)
                 if dial.exec():
                     node = node._replace(dict=dial.get_values())
                 else:
@@ -644,7 +643,7 @@ class MeasurementSettings(QWidget):
         node = deepcopy(selected_item.stat)
         # noinspection PyTypeChecker
         node = self.get_parameters(
-            node, self.measurement_area_choose.get_value(), self.per_component.get_value(), self.power_num.value()
+            node, self.measurement_area_choose.currentEnum(), self.per_component.currentEnum(), self.power_num.value()
         )
         if node is None:
             return
@@ -803,9 +802,9 @@ class SegAdvancedWindow(AdvancedWindow):
     """
 
     def __init__(self, settings, parent=None, reload_list=None):
-        super().__init__(settings, ["result_image", "raw_image"], parent)
+        super().__init__(settings, ["result_image", "raw_image"], reload_list, parent)
         self.settings = settings
-        self.reload_list = reload_list if reload_list is not None else []
+
         self.setWindowTitle("Settings and Measurement")
         self.advanced_settings = Properties(settings)
         # self.colormap_settings = ColorSelector(settings, ["result_control"])
@@ -847,7 +846,7 @@ class MultipleInput(QDialog):
             return res
 
         field_dict = {str: QLineEdit, float: create_input_float, int: create_input_int}
-        QDialog.__init__(self)
+        super().__init__()
         ok_butt = QPushButton("Ok", self)
         cancel_butt = QPushButton("Cancel", self)
         self.object_dict = {}
