@@ -4,11 +4,12 @@ import os
 import typing
 from io import BytesIO
 from pathlib import Path
+from pickle import PicklingError  # nosec
 
 import numpy as np
 import sentry_sdk
 from qtpy.QtCore import QByteArray, Qt, QTimer
-from qtpy.QtGui import QIcon
+from qtpy.QtGui import QIcon, QStandardItem, QStandardItemModel
 from qtpy.QtWidgets import (
     QDialog,
     QFileDialog,
@@ -16,7 +17,7 @@ from qtpy.QtWidgets import (
     QHBoxLayout,
     QLabel,
     QLineEdit,
-    QListWidget,
+    QListView,
     QListWidgetItem,
     QMessageBox,
     QProgressBar,
@@ -39,9 +40,9 @@ from PartSegCore.universal_const import Units
 from PartSegData import icons_dir
 
 from .. import parsed_version
-from ..common_gui.custom_save_dialog import SaveDialog
+from ..common_gui.custom_save_dialog import CustomSaveDialog
 from ..common_gui.error_report import ExceptionList, ExceptionListItem
-from ..common_gui.searchable_combo_box import SearchCombBox
+from ..common_gui.searchable_combo_box import SearchComboBox
 from ..common_gui.select_multiple_files import AddFiles
 from ..common_gui.universal_gui_part import Spacing, right_label
 from .partseg_settings import PartSettings
@@ -81,7 +82,8 @@ class ProgressView(QWidget):
     """
 
     def __init__(self, parent, batch_manager):
-        QWidget.__init__(self, parent)
+        super().__init__(parent)
+        self.task_count = 0
         self.calculation_manager = batch_manager
         self.whole_progress = QProgressBar(self)
         self.whole_progress.setMinimum(0)
@@ -94,9 +96,13 @@ class ProgressView(QWidget):
         self.part_progress.setFormat("%v of %m")
         self.whole_label = QLabel("All batch progress:", self)
         self.part_label = QLabel("Single batch progress:", self)
+        self.cancel_remove_btn = QPushButton("Remove task")
+        self.cancel_remove_btn.setDisabled(True)
         self.logs = ExceptionList(self)
         self.logs.setToolTip("Logs")
-        self.task_que = QListWidget()
+        self.task_view = QListView()
+        self.task_que = QStandardItemModel(self)
+        self.task_view.setModel(self.task_que)
         self.process_num_timer = QTimer()
         self.process_num_timer.setInterval(1000)
         self.process_num_timer.setSingleShot(True)
@@ -106,6 +112,7 @@ class ProgressView(QWidget):
         self.number_of_process.setValue(1)
         self.number_of_process.setToolTip("Number of process used in batch calculation")
         self.number_of_process.valueChanged.connect(self.process_num_timer_start)
+        self.progress_item_dict = {}
         layout = QGridLayout()
         layout.addWidget(self.whole_label, 0, 0, Qt.AlignRight)
         layout.addWidget(self.whole_progress, 0, 1, 1, 2)
@@ -115,14 +122,38 @@ class ProgressView(QWidget):
         lab.setToolTip("Number of process used in batch calculation")
         layout.addWidget(lab, 2, 0)
         layout.addWidget(self.number_of_process, 2, 1)
-        layout.addWidget(self.logs, 3, 0, 1, 3)
-        layout.addWidget(self.task_que, 0, 4, 0, 1)
+        layout.addWidget(self.logs, 3, 0, 2, 3)
+        layout.addWidget(self.task_view, 0, 4, 4, 1)
+        layout.addWidget(self.cancel_remove_btn, 4, 4, 1, 1)
         layout.setColumnMinimumWidth(2, 10)
         layout.setColumnStretch(2, 1)
         self.setLayout(layout)
         self.preview_timer = QTimer()
         self.preview_timer.setInterval(1000)
         self.preview_timer.timeout.connect(self.update_info)
+        self.task_view.selectionModel().currentChanged.connect(self.task_selection_change)
+        self.cancel_remove_btn.clicked.connect(self.task_cancel_remove)
+
+    def task_selection_change(self, new, old):
+        task: CalculationProcessItem = self.task_que.item(new.row(), new.column())
+        if task is None:
+            self.cancel_remove_btn.setDisabled(True)
+            return
+        self.cancel_remove_btn.setEnabled(True)
+        if task.is_finished():
+            self.cancel_remove_btn.setText(f"Remove task {task.num}")
+        else:
+            self.cancel_remove_btn.setText(f"Cancel task {task.num}")
+
+    def task_cancel_remove(self):
+        index = self.task_view.selectionModel().currentIndex()
+        task: CalculationProcessItem = self.task_que.item(index.row(), index.column())
+        if task.is_finished():
+            self.calculation_manager.remove_calculation(task.calculation)
+            self.task_que.takeRow(index.row())
+        else:
+            self.calculation_manager.cancel_calculation(task.calculation)
+        print(task)
 
     def new_task(self):
         self.whole_progress.setMaximum(self.calculation_manager.calculation_size)
@@ -147,16 +178,22 @@ class ProgressView(QWidget):
                     sentry_sdk.capture_event(el[1][1][0])
         self.whole_progress.setValue(res.global_counter)
         working_search = True
-        for i, (progress, total) in enumerate(res.jobs_status):
+        for uuid, progress in res.jobs_status.items():
+            calculation = self.calculation_manager.calculation_dict[uuid]
+            total = len(calculation.file_list)
+            if uuid in self.progress_item_dict:
+                item = self.progress_item_dict[uuid]
+                item.update_count(progress)
+            else:
+                item = CalculationProcessItem(calculation, self.task_count, progress)
+                self.task_count += 1
+                self.task_que.appendRow(item)
+                self.progress_item_dict[uuid] = item
+
             if working_search and progress != total:
                 self.part_progress.setMaximum(total)
                 self.part_progress.setValue(progress)
                 working_search = False
-            if i < self.task_que.count():
-                item = self.task_que.item(i)
-                item.setText(f"Task {i} ({progress}/{total})")
-            else:
-                self.task_que.addItem(f"Task {i} ({progress}/{total})")
         if not self.calculation_manager.has_work:
             self.part_progress.setValue(self.part_progress.maximum())
             self.preview_timer.stop()
@@ -185,7 +222,7 @@ class FileChoose(QWidget):
     """
 
     def __init__(self, settings: PartSettings, batch_manager, parent=None):
-        QWidget.__init__(self, parent)
+        super().__init__(parent)
         self.files_to_proceed = set()
         self.settings = settings
         self.batch_manager = batch_manager
@@ -193,7 +230,7 @@ class FileChoose(QWidget):
         self.progress = ProgressView(self, batch_manager)
         self.run_button = QPushButton("Process")
         self.run_button.setDisabled(True)
-        self.calculation_choose = SearchCombBox()
+        self.calculation_choose = SearchComboBox()
         self.calculation_choose.addItem("<no workflow>")
         self.calculation_choose.currentIndexChanged[str].connect(self.change_situation)
         self.result_file = QLineEdit(self)
@@ -224,8 +261,14 @@ class FileChoose(QWidget):
             self.files_widget.get_paths(), plan, str(self.result_file.text()), self.settings, self.batch_manager
         )
         if dial.exec_():
-            self.batch_manager.add_calculation(dial.get_data())
-            self.progress.new_task()
+            try:
+                self.batch_manager.add_calculation(dial.get_data())
+                self.progress.new_task()
+            except PicklingError as e:
+                if state_store.develop:
+                    QMessageBox.warning(self, "Pickle error", "Please restart PartSeg.")
+                else:
+                    raise e
 
     def showEvent(self, _):
         current_calc = str(self.calculation_choose.currentText())
@@ -255,9 +298,7 @@ class FileChoose(QWidget):
             self.files_widget.mask_list = []
 
     def chose_result_file(self):
-        dial = SaveDialog(
-            {SaveExcel.get_short_name(): SaveExcel}, system_widget=False, history=self.settings.get_path_history()
-        )
+        dial = CustomSaveDialog(SaveExcel, system_widget=False, history=self.settings.get_path_history())
         dial.setDirectory(
             self.settings.get("io.save_directory", self.settings.get("io.open_directory", str(Path.home())))
         )
@@ -540,3 +581,21 @@ class CalculationPrepare(QDialog):
                 widget.setIcon(0, warn_icon)
             else:
                 widget.setIcon(0, bad_icon)
+
+
+class CalculationProcessItem(QStandardItem):
+    def __init__(self, calculation: Calculation, num: int, count, *args, **kwargs):
+        text = f"Task {num} ({count}/{len(calculation.file_list)})"
+        super().__init__(text, *args, **kwargs)
+        self.calculation = calculation
+        self.num = num
+        self.count = count
+        self.setToolTip(str(calculation.calculation_plan))
+        self.setEditable(False)
+
+    def update_count(self, count):
+        self.count = count
+        self.setText(f"Task {self.num} ({count}/{len(self.calculation.file_list)})")
+
+    def is_finished(self) -> bool:
+        return self.count == len(self.calculation.file_list)
