@@ -1,5 +1,4 @@
 import os
-import sys
 from pathlib import Path
 from typing import Type
 
@@ -36,13 +35,15 @@ from PartSegCore.roi_info import ROIInfo
 from PartSegCore.segmentation.algorithm_base import ROIExtractionResult
 from PartSegImage import TiffImageReader
 
+from ..common_backend.except_hook import show_warning
 from ..common_gui.algorithms_description import AlgorithmChoose, InteractiveAlgorithmSettingsWidget
 from ..common_gui.channel_control import ChannelProperty
-from ..common_gui.custom_save_dialog import SaveDialog
+from ..common_gui.custom_save_dialog import CustomSaveDialog
 from ..common_gui.equal_column_layout import EqualColumnLayout
+from ..common_gui.exception_hooks import OPEN_ERROR, load_data_exception_hook
 from ..common_gui.mask_widget import MaskDialogBase
 from ..common_gui.multiple_file_widget import MultipleFileWidget
-from ..common_gui.searchable_combo_box import SearchCombBox
+from ..common_gui.searchable_combo_box import SearchComboBox
 from ..common_gui.stack_image_view import ColorBar
 from ..common_gui.universal_gui_part import TextShow
 from ..common_gui.waiting_dialog import ExecuteFunctionDialog, WaitingDialog
@@ -79,17 +80,17 @@ class Options(QWidget):
         self.save_pipe_btn = QPushButton("Save pipeline")
         self.save_pipe_btn.clicked.connect(self.save_pipeline)
         self.save_pipe_btn.setToolTip("Save current pipeline. Last element is last executed algorithm")
-        self.choose_pipe = SearchCombBox()
+        self.choose_pipe = SearchComboBox()
         self.choose_pipe.addItem("<none>")
-        self.choose_pipe.addItems(list(self._settings.segmentation_pipelines.keys()))
+        self.choose_pipe.addItems(list(self._settings.roi_pipelines.keys()))
         self.choose_pipe.textActivated.connect(self.choose_pipeline)
         self.choose_pipe.setToolTip("Execute chosen pipeline")
         self.save_profile_btn = QPushButton("Save profile")
         self.save_profile_btn.setToolTip("Save values from current view")
         self.save_profile_btn.clicked.connect(self.save_profile)
-        self.choose_profile = SearchCombBox()
+        self.choose_profile = SearchComboBox()
         self.choose_profile.addItem("<none>")
-        self.choose_profile.addItems(list(self._settings.segmentation_profiles.keys()))
+        self.choose_profile.addItems(list(self._settings.roi_profiles.keys()))
         self.choose_profile.setToolTip("Select profile to restore its settings. Execute if interactive is checked")
         # image state
         self.compare_btn = QPushButton("Compare")
@@ -181,7 +182,7 @@ class Options(QWidget):
             text, ok = QInputDialog.getText(self, "Pipeline name", "Input pipeline name here")
             if not ok:
                 return
-            if text in self._settings.segmentation_pipelines and QMessageBox.No == QMessageBox.warning(
+            if text in self._settings.roi_pipelines and QMessageBox.No == QMessageBox.warning(
                 self,
                 "Already exists",
                 "Profile with this name already exist. Overwrite?",
@@ -190,7 +191,7 @@ class Options(QWidget):
             ):
                 continue
             profile = SegmentationPipeline(name=text, segmentation=current_segmentation, mask_history=mask_history)
-            self._settings.segmentation_pipelines[text] = profile
+            self._settings.roi_pipelines[text] = profile
             self._settings.dump()
             self.choose_pipe.addItem(text)
             break
@@ -198,7 +199,7 @@ class Options(QWidget):
     def choose_pipeline(self, text):
         if text == "<none>":
             return
-        pipeline = self._settings.segmentation_pipelines[text]
+        pipeline = self._settings.roi_pipelines[text]
         process_thread = CalculatePipelineThread(self._settings.image, self._settings.mask, pipeline)
         dial = WaitingDialog(process_thread)
 
@@ -216,14 +217,14 @@ class Options(QWidget):
             if self.choose_profile.itemData(i, Qt.ToolTipRole) is not None:
                 continue
             text = self.choose_profile.itemText(i)
-            profile: ROIExtractionProfile = self._settings.segmentation_profiles[text]
+            profile: ROIExtractionProfile = self._settings.roi_profiles[text]
             tool_tip_text = str(profile)
             self.choose_profile.setItemData(i, tool_tip_text, Qt.ToolTipRole)
         for i in range(1, self.choose_pipe.count()):
             if self.choose_pipe.itemData(i, Qt.ToolTipRole) is not None:
                 continue
             text = self.choose_pipe.itemText(i)
-            profile: SegmentationPipeline = self._settings.segmentation_pipelines[text]
+            profile: SegmentationPipeline = self._settings.roi_pipelines[text]
             tool_tip_text = str(profile)
             self.choose_pipe.setItemData(i, tool_tip_text, Qt.ToolTipRole)
 
@@ -246,9 +247,9 @@ class Options(QWidget):
     def event(self, event: QEvent):
         if event.type() == QEvent.WindowActivate:
             # update combobox for segmentation
-            self.update_combo_box(self.choose_profile, self._settings.segmentation_profiles)
+            self.update_combo_box(self.choose_profile, self._settings.roi_profiles)
             # update combobox for pipeline
-            self.update_combo_box(self.choose_pipe, self._settings.segmentation_pipelines)
+            self.update_combo_box(self.choose_pipe, self._settings.roi_pipelines)
             self.update_tooltips()
         return super().event(event)
 
@@ -262,7 +263,7 @@ class Options(QWidget):
             text, ok = QInputDialog.getText(self, "Profile Name", "Input profile name here")
             if not ok:
                 return
-            if text in self._settings.segmentation_profiles and QMessageBox.No == QMessageBox.warning(
+            if text in self._settings.roi_profiles and QMessageBox.No == QMessageBox.warning(
                 self,
                 "Already exists",
                 "Profile with this name already exist. Overwrite?",
@@ -271,7 +272,7 @@ class Options(QWidget):
             ):
                 continue
             resp = ROIExtractionProfile(text, widget.name, widget.get_values())
-            self._settings.segmentation_profiles[text] = resp
+            self._settings.roi_profiles[text] = resp
             self._settings.dump()
             self.choose_profile.addItem(text)
             self.update_tooltips()
@@ -283,7 +284,7 @@ class Options(QWidget):
             return
         interactive = self.interactive_use.isChecked()
         self.interactive_use.setChecked(False)
-        profile = self._settings.segmentation_profiles[val]
+        profile = self._settings.roi_profiles[val]
         self.algorithm_choose_widget.change_algorithm(profile.algorithm, profile.values)
         self.choose_profile.blockSignals(True)
         self.choose_profile.setCurrentIndex(0)
@@ -392,7 +393,7 @@ class MainMenu(BaseMainMenu):
 
     def save_file(self):
         base_values = self.settings.get("save_parameters", {})
-        dial = SaveDialog(
+        dial = CustomSaveDialog(
             save_dict, system_widget=False, base_values=base_values, history=self.settings.get_path_history()
         )
         dial.selectFile(os.path.splitext(os.path.basename(self.settings.image_path))[0])
@@ -410,13 +411,8 @@ class MainMenu(BaseMainMenu):
             base_values[selected_filter] = values
 
             def exception_hook(exception):
-                from qtpy.QtCore import QMetaObject
-                from qtpy.QtWidgets import QApplication
-
-                instance = QApplication.instance()
                 if isinstance(exception, ValueError):
-                    instance.warning = "Save error", f"Error during saving\n{exception}"
-                    QMetaObject.invokeMethod(instance, "show_warning", Qt.QueuedConnection)
+                    show_warning("Save error", f"Error during saving\n{exception}")
                 else:
                     raise exception
 
@@ -434,34 +430,13 @@ class MainMenu(BaseMainMenu):
 
     def load_data(self):
         def exception_hook(exception):
-            from qtpy.QtCore import QMetaObject
-            from qtpy.QtWidgets import QApplication
-
-            instance = QApplication.instance()
-            if isinstance(exception, ValueError) and exception.args[0] == "Incompatible shape of mask and image":
-                instance.warning = (
-                    "Open error",
-                    "Most probably you try to load mask from other image. Check selected files",
-                )
-                QMetaObject.invokeMethod(instance, "show_warning", Qt.QueuedConnection)
-            elif isinstance(exception, MemoryError):
-                instance.warning = "Open error", f"Not enough memory to read this image: {exception}"
-                QMetaObject.invokeMethod(instance, "show_warning", Qt.QueuedConnection)
-            elif isinstance(exception, IOError):
-                instance.warning = "Open error", f"Some problem with reading from disc: {exception}"
-                QMetaObject.invokeMethod(instance, "show_warning", Qt.QueuedConnection)
-            elif isinstance(exception, KeyError):
-                instance.warning = "Open error", f"Some problem project file: {exception}"
-                QMetaObject.invokeMethod(instance, "show_warning", Qt.QueuedConnection)
-                print(exception, file=sys.stderr)
-            elif isinstance(exception, WrongFileTypeException):
-                instance.warning = (
-                    "Open error",
+            if isinstance(exception, WrongFileTypeException):
+                show_warning(
+                    OPEN_ERROR,
                     "No needed files inside archive. Most probably you choose file from segmentation mask",
                 )
-                QMetaObject.invokeMethod(instance, "show_warning", Qt.QueuedConnection)
             else:
-                raise exception
+                load_data_exception_hook(exception)
 
         try:
             dial = CustomLoadDialog(load_functions.load_dict, history=self.settings.get_path_history())
@@ -607,6 +582,7 @@ class MainWindow(BaseMainWindow):
         view_menu.addAction("Napari viewer").triggered.connect(self.napari_viewer_show)
         view_menu.addAction("Toggle Multiple Files").triggered.connect(self.toggle_multiple_files)
         view_menu.addAction("Toggle left panel").triggered.connect(self.toggle_left_panel)
+        view_menu.addAction("Toggle console").triggered.connect(self._toggle_console)
         action = view_menu.addAction("Screenshot right panel")
         action.triggered.connect(self.screenshot(self.result_image))
         action.setShortcut(QKeySequence.Print)
@@ -674,7 +650,6 @@ class MainWindow(BaseMainWindow):
                     return
             self.batch_window.close()
         self.advanced_window.close()
-        self.settings.dump()
         del self.batch_window
         del self.advanced_window
         super().closeEvent(event)
