@@ -31,8 +31,8 @@ from qtpy.QtWidgets import (
     QVBoxLayout,
     QWidget,
 )
+from superqt import QEnumComboBox
 
-from PartSeg.common_gui.universal_gui_part import EnumComboBox
 from PartSegCore.algorithm_describe_base import AlgorithmProperty, ROIExtractionProfile
 from PartSegCore.analysis.algorithm_description import analysis_algorithm_dict
 from PartSegCore.analysis.calculation_plan import (
@@ -234,7 +234,7 @@ class FileMask(QWidget):
         if self.select_type.currentIndex() == 0:
             return self.first_text.text().strip() != ""
         if self.select_type.currentIndex() == 1:
-            return self.first_text.text().strip() != "" and self.second_text.text().strip() != ""
+            return "" not in {self.first_text.text().strip(), self.second_text.text().strip()}
 
         text = self.first_text.text().strip()
         return text != "" and os.path.exists(text) and os.path.isfile(text)
@@ -275,7 +275,7 @@ class CreatePlan(QWidget):
         self.clean_plan_btn = QPushButton("Remove all")
         self.remove_btn = QPushButton("Remove")
         self.update_element_chk = QCheckBox("Update element")
-        self.change_root = EnumComboBox(RootType)
+        self.change_root = QEnumComboBox(enum_class=RootType)
         self.save_choose = QComboBox()
         self.save_choose.addItem("<none>")
         self.save_choose.addItems(list(self.save_translate_dict.keys()))
@@ -288,15 +288,15 @@ class CreatePlan(QWidget):
         self.generate_mask_btn = QPushButton("Add mask")
         self.generate_mask_btn.setToolTip("Mask need to have unique name")
         self.mask_name = QLineEdit()
-        self.mask_operation = EnumComboBox(MaskOperation)
+        self.mask_operation = QEnumComboBox(enum_class=MaskOperation)
 
         self.chanel_num = QSpinBox()
         self.choose_channel_for_measurements = QComboBox()
         self.choose_channel_for_measurements.addItems(
             ["Same as segmentation"] + [str(x + 1) for x in range(MAX_CHANNEL_NUM)]
         )
-        self.units_choose = EnumComboBox(Units)
-        self.units_choose.set_value(self.settings.get("units_value", Units.nm))
+        self.units_choose = QEnumComboBox(enum_class=Units)
+        self.units_choose.setCurrentEnum(self.settings.get("units_value", Units.nm))
         self.chanel_num.setRange(0, 10)
         self.expected_node_type = None
         self.save_constructor = None
@@ -456,7 +456,7 @@ class CreatePlan(QWidget):
         self.node_type_changed()
 
     def change_root_type(self):
-        value: RootType = self.change_root.get_value()
+        value: RootType = self.change_root.currentEnum()
         self.calculation_plan.set_root_type(value)
         self.plan.update_view()
 
@@ -580,9 +580,13 @@ class CreatePlan(QWidget):
         if text != "" and text in self.mask_set:
             QMessageBox.warning(self, "Already exists", "Mask with this name already exists", QMessageBox.Ok)
             return
-        if _check_widget(self.mask_stack, EnumComboBox):  # existing mask
+        if _check_widget(self.mask_stack, MaskWidget):
+            mask_ob = MaskCreate(text, self.segmentation_mask.get_mask_property())
+        elif _check_widget(self.mask_stack, FileMask):
+            mask_ob = self.file_mask.get_value(text)
+        elif _check_widget(self.mask_stack, QEnumComboBox):  # existing mask
             mask_dialog = TwoMaskDialog
-            if self.mask_operation.get_value() == MaskOperation.mask_intersection:  # Mask intersection
+            if self.mask_operation.currentEnum() == MaskOperation.mask_intersection:  # Mask intersection
                 MaskConstruct = MaskIntersection
             else:
                 MaskConstruct = MaskSum
@@ -592,10 +596,6 @@ class CreatePlan(QWidget):
             names = dial.get_result()
 
             mask_ob = MaskConstruct(text, *names)
-        elif _check_widget(self.mask_stack, MaskWidget):
-            mask_ob = MaskCreate(text, self.segmentation_mask.get_mask_property())
-        elif _check_widget(self.mask_stack, FileMask):
-            mask_ob = self.file_mask.get_value(text)
         else:
             raise ValueError("Unknowsn widget")
 
@@ -634,7 +634,19 @@ class CreatePlan(QWidget):
         ):
             self.generate_mask_btn.setDisabled(True)
             return
-        if _check_widget(self.mask_stack, EnumComboBox):  # reuse mask
+        if _check_widget(self.mask_stack, MaskWidget):  # mask from segmentation
+            if (not update and node_type == NodeType.segment) or (update and node_type == NodeType.mask):
+                self.generate_mask_btn.setEnabled(True)
+            else:
+                self.generate_mask_btn.setEnabled(False)
+            self.generate_mask_btn.setToolTip("Select segmentation")
+        elif _check_widget(self.mask_stack, FileMask):
+            if (not update and node_type == NodeType.root) or (update and node_type == NodeType.file_mask):
+                self.generate_mask_btn.setEnabled(self.file_mask.is_valid())
+            else:
+                self.generate_mask_btn.setEnabled(False)
+            self.generate_mask_btn.setToolTip("Need root selected")
+        else:  # reuse mask
             if len(self.mask_set) > 1 and (
                 (not update and node_type == NodeType.root) or (update and node_type == NodeType.file_mask)
             ):
@@ -642,18 +654,6 @@ class CreatePlan(QWidget):
             else:
                 self.generate_mask_btn.setEnabled(False)
             self.generate_mask_btn.setToolTip("Need at least two named mask and root selected")
-        elif _check_widget(self.mask_stack, MaskWidget):  # mask from segmentation
-            if (not update and node_type == NodeType.segment) or (update and node_type == NodeType.mask):
-                self.generate_mask_btn.setEnabled(True)
-            else:
-                self.generate_mask_btn.setEnabled(False)
-            self.generate_mask_btn.setToolTip("Select segmentation")
-        else:
-            if (not update and node_type == NodeType.root) or (update and node_type == NodeType.file_mask):
-                self.generate_mask_btn.setEnabled(self.file_mask.is_valid())
-            else:
-                self.generate_mask_btn.setEnabled(False)
-            self.generate_mask_btn.setToolTip("Need root selected")
 
     def mask_name_changed(self, text):
         if str(text) in self.mask_set:
@@ -670,17 +670,17 @@ class CreatePlan(QWidget):
     def add_segmentation(self):
         if self.segment_stack.currentIndex() == 0:
             text = str(self.segment_profile.currentItem().text())
-            if text not in self.settings.segmentation_profiles:
+            if text not in self.settings.roi_profiles:
                 self.refresh_all_profiles()
                 return
-            profile = self.settings.segmentation_profiles[text]
+            profile = self.settings.roi_profiles[text]
             if self.update_element_chk.isChecked():
                 self.calculation_plan.replace_step(profile)
             else:
                 self.calculation_plan.add_step(profile)
         else:  # self.segment_stack.currentIndex() == 1
             text = self.pipeline_profile.currentItem().text()
-            segmentation_pipeline = self.settings.segmentation_pipelines[text]
+            segmentation_pipeline = self.settings.roi_pipelines[text]
             pos = self.calculation_plan.current_pos[:]
             old_pos = self.calculation_plan.current_pos[:]
             for el in segmentation_pipeline.mask_history:
@@ -709,7 +709,7 @@ class CreatePlan(QWidget):
             channel=channel,
             measurement_profile=measurement_copy,
             name_prefix=prefix,
-            units=self.units_choose.get_value(),
+            units=self.units_choose.currentEnum(),
         )
         if self.update_element_chk.isChecked():
             self.calculation_plan.replace_step(measurement_calculate)
@@ -737,17 +737,16 @@ class CreatePlan(QWidget):
         name = str(self.mask_name.text()).strip()
         self.generate_mask_btn.setDisabled(True)
         # load mask from file
-        if not self.update_element_chk.isChecked():
-            # generate mask from segmentation
-            if self.mask_allow and (name == "" or name not in self.mask_set):
-                self.generate_mask_btn.setEnabled(True)
-        else:
+        if self.update_element_chk.isChecked():
             if self.node_type not in [NodeType.file_mask, NodeType.mask]:
                 return
             # generate mask from segmentation
             if self.node_type == NodeType.mask and (name == "" or name == self.node_name or name not in self.mask_set):
                 self.generate_mask_btn.setEnabled(True)
 
+        elif self.mask_allow and (name == "" or name not in self.mask_set):
+            self.generate_mask_btn.setEnabled(True)
+
     def add_calculation_plan(self, text=None):
         if text is None or isinstance(text, bool):
             text, ok = QInputDialog.getText(self, "Plan title", "Set plan title")
@@ -801,8 +800,8 @@ class CreatePlan(QWidget):
 
     def refresh_all_profiles(self):
         new_measurements = list(sorted(self.settings.measurement_profiles.keys()))
-        new_segment = list(sorted(self.settings.segmentation_profiles.keys()))
-        new_pipelines = list(sorted(self.settings.segmentation_pipelines.keys()))
+        new_segment = list(sorted(self.settings.roi_profiles.keys()))
+        new_pipelines = list(sorted(self.settings.roi_pipelines.keys()))
         measurement_index = self.get_index(self.measurements_list.currentItem(), new_measurements)
         segment_index = self.get_index(self.segment_profile.currentItem(), new_segment)
         pipeline_index = self.get_index(self.pipeline_profile.currentItem(), new_pipelines)
@@ -829,11 +828,10 @@ class CreatePlan(QWidget):
                 self.add_calculation_btn.setEnabled(True)
             else:
                 self.add_calculation_btn.setDisabled(True)
+        elif self.measurements_list.currentItem() is not None:
+            self.add_calculation_btn.setEnabled(self.mask_allow)
         else:
-            if self.measurements_list.currentItem() is not None:
-                self.add_calculation_btn.setEnabled(self.mask_allow)
-            else:
-                self.add_calculation_btn.setDisabled(True)
+            self.add_calculation_btn.setDisabled(True)
 
     def show_segment_info(self, text=None):
         if self.protect:
@@ -846,14 +844,14 @@ class CreatePlan(QWidget):
                     text = str(self.segment_profile.currentItem().text())
                 else:
                     return
-            profile = self.settings.segmentation_profiles[text]
+            profile = self.settings.roi_profiles[text]
         else:
             if text is None:
                 if self.pipeline_profile.currentItem() is not None:
                     text = str(self.pipeline_profile.currentItem().text())
                 else:
                     return
-            profile = self.settings.segmentation_pipelines[text]
+            profile = self.settings.roi_pipelines[text]
         self.information.setText(profile.pretty_print(analysis_algorithm_dict))
 
     def show_segment(self):
@@ -1073,9 +1071,6 @@ class CalculateInfo(QWidget):
         self.calculate_plans.addItems(new_plan_list)
         if index != -1:
             self.calculate_plans.setCurrentRow(index)
-        else:
-            pass
-            # self.plan_view.setText("")
         self.protect = False
 
     def export_plans(self):
