@@ -1,5 +1,4 @@
 import re
-import sys
 from collections import defaultdict
 from functools import partial
 from os import path
@@ -13,12 +12,7 @@ from PartSegCore.algorithm_describe_base import ROIExtractionProfile
 from PartSegCore.mask.algorithm_description import mask_algorithm_dict
 from PartSegCore.mask.io_functions import LoadROIImage, LoadStackImage, MaskProjectTuple, SaveROI
 from PartSegCore.segmentation import StackAlgorithm
-from PartSegCore.segmentation.algorithm_base import SegmentationAlgorithm
-
-if sys.version_info.minor == 6:
-    SegmentationTupleWrapper = object
-else:
-    SegmentationTupleWrapper = MaskProjectTuple
+from PartSegCore.segmentation.algorithm_base import ROIExtractionAlgorithm
 
 
 class BatchTask(NamedTuple):
@@ -32,8 +26,8 @@ class BatchProceed(QThread):
     progress_signal = Signal(str, int, str, int)
     range_signal = Signal(int, int)
     execution_done = Signal()
-    multiple_result = Signal(SegmentationTupleWrapper)
-    algorithm: SegmentationAlgorithm
+    multiple_result = Signal(MaskProjectTuple)
+    algorithm: ROIExtractionAlgorithm
 
     def __init__(self):
         super().__init__()
