@@ -25,6 +25,7 @@ from qtpy.QtWidgets import (
     QVBoxLayout,
     QWidget,
 )
+from superqt import QEnumComboBox
 
 import PartSegData
 from PartSegCore import UNIT_SCALE, Units, state_store
@@ -34,14 +35,17 @@ from PartSegCore.mask.algorithm_description import mask_algorithm_dict
 from PartSegCore.mask.history_utils import create_history_element_from_segmentation_tuple
 from PartSegCore.mask.io_functions import LoadROI, LoadROIFromTIFF, LoadROIParameters, MaskProjectTuple, SaveROI
 from PartSegCore.project_info import HistoryElement, HistoryProblem, calculate_mask_from_project
+from PartSegCore.roi_info import ROIInfo
 from PartSegImage import Image, TiffImageReader
 
 from .._roi_mask.segmentation_info_dialog import SegmentationInfoDialog
+from ..common_backend.base_settings import ROI_NOT_FIT
 from ..common_gui.advanced_tabs import AdvancedWindow
-from ..common_gui.algorithms_description import AlgorithmChoose, AlgorithmSettingsWidget, EnumComboBox
+from ..common_gui.algorithms_description import AlgorithmChoose, AlgorithmSettingsWidget
 from ..common_gui.channel_control import ChannelProperty
 from ..common_gui.custom_load_dialog import CustomLoadDialog
-from ..common_gui.custom_save_dialog import SaveDialog
+from ..common_gui.custom_save_dialog import CustomSaveDialog
+from ..common_gui.exception_hooks import load_data_exception_hook
 from ..common_gui.flow_layout import FlowLayout
 from ..common_gui.main_window import BaseMainMenu, BaseMainWindow
 from ..common_gui.mask_widget import MaskDialogBase
@@ -86,9 +90,10 @@ class MaskDialog(MaskDialogBase):
         history.arrays.seek(0)
         seg = np.load(history.arrays)
         history.arrays.seek(0)
-        self.settings.roi = seg["segmentation"]
-        self.settings.set_segmentation(
-            seg["segmentation"],
+        # TODO Check me
+        # self.settings.roi = seg["segmentation"]
+        self.settings._set_roi_info(  # pylint: disable=W0212
+            ROIInfo(seg["segmentation"]),
             False,
             history.roi_extraction_parameters["selected"],
             history.roi_extraction_parameters["parameters"],
@@ -160,11 +165,12 @@ class MainMenu(BaseMainMenu):
 
     def show_advanced_window(self):
         if self.advanced_window is None:
-            self.advanced_window = AdvancedWindow(self.settings, ["channelcontrol"])
-            # FIXME temporary workaround
-            self.advanced_window.reload_list = []
+            self.advanced_window = AdvancedWindow(self.settings, ["channelcontrol"], reload_list=[self.reload])
         self.advanced_window.show()
 
+    def reload(self):
+        self.parent().parent().options_panel.algorithm_options.algorithm_choose_widget.reload(mask_algorithm_dict)
+
     def load_image(self):
         # TODO move segmentation with image load to load_segmentaion
         dial = CustomLoadDialog(io_functions.load_dict)
@@ -182,28 +188,12 @@ class MainMenu(BaseMainMenu):
         self.settings.set("io.load_data_filter", load_property.selected_filter)
         self.settings.add_load_files_history(load_property.load_location, load_property.load_class.get_name())
 
-        def exception_hook(exception):
-            if isinstance(exception, ValueError) and exception.args[0] == "not a TIFF file":
-                QMessageBox.warning(self, "Open error", "Image is not proper tiff/lsm image")
-            elif isinstance(exception, MemoryError):
-                QMessageBox.warning(self, "Open error", "Not enough memory to read this image")
-            elif isinstance(exception, IOError):
-                QMessageBox.warning(self, "Open error", f"Some problem with reading from disc: {exception}")
-            elif isinstance(exception, WrongFileTypeException):
-                QMessageBox.warning(
-                    self,
-                    "Open error",
-                    "No needed files inside archive. Most probably you choose file from segmentation analysis",
-                )
-            else:
-                raise exception
-
         execute_dialog = ExecuteFunctionDialog(
             load_property.load_class.load,
             [load_property.load_location],
             {"metadata": {"default_spacing": self.settings.image.spacing}},
             text="Load data",
-            exception_hook=exception_hook,
+            exception_hook=load_data_exception_hook,
         )
         if execute_dialog.exec():
             result = execute_dialog.get_result()
@@ -283,7 +273,7 @@ class MainMenu(BaseMainMenu):
                     self.settings.set_project_info(dial.get_result())
                     return
                 except ValueError as e:
-                    if e.args != ("Segmentation do not fit to image",):
+                    if e.args != (ROI_NOT_FIT,):
                         raise
                     self.segmentation_dialog.set_additional_text(
                         "Segmentation do not fit to image, maybe you would lie to load parameters only."
@@ -305,7 +295,7 @@ class MainMenu(BaseMainMenu):
         if self.settings.roi is None:
             QMessageBox.warning(self, "No segmentation", "No segmentation to save")
             return
-        dial = SaveDialog(io_functions.save_segmentation_dict, False, history=self.settings.get_path_history())
+        dial = CustomSaveDialog(io_functions.save_segmentation_dict, False, history=self.settings.get_path_history())
         dial.setDirectory(self.settings.get("io.save_segmentation_directory", str(Path.home())))
         dial.selectFile(os.path.splitext(os.path.basename(self.settings.image_path))[0] + ".seg")
         if not dial.exec_():
@@ -337,7 +327,7 @@ class MainMenu(BaseMainMenu):
         if self.settings.roi is None or len(self.settings.sizes) == 1:
             QMessageBox.warning(self, "No components", "No components to save")
             return
-        dial = SaveDialog(
+        dial = CustomSaveDialog(
             io_functions.save_components_dict,
             False,
             history=self.settings.get_path_history(),
@@ -491,8 +481,8 @@ class AlgorithmOptions(QWidget):
         control_view = image_view.get_control_view()
         super().__init__()
         self.settings = settings
-        self.show_result = EnumComboBox(LabelEnum)  # QCheckBox("Show result")
-        self.show_result.set_value(control_view.show_label)
+        self.show_result = QEnumComboBox(enum_class=LabelEnum)  # QCheckBox("Show result")
+        self.show_result.setCurrentEnum(control_view.show_label)
         self.opacity = QDoubleSpinBox()
         self.opacity.setRange(0, 1)
         self.opacity.setSingleStep(0.1)
@@ -601,7 +591,7 @@ class AlgorithmOptions(QWidget):
         # noinspection PyUnresolvedReferences
         self.opacity.valueChanged.connect(control_view.set_opacity)
         # noinspection PyUnresolvedReferences
-        self.show_result.current_choose.connect(control_view.set_show_label)
+        self.show_result.currentEnumChanged.connect(control_view.set_show_label)
         self.only_borders.stateChanged.connect(control_view.set_borders)
         # noinspection PyUnresolvedReferences
         self.borders_thick.valueChanged.connect(control_view.set_borders_thick)
@@ -615,7 +605,7 @@ class AlgorithmOptions(QWidget):
         self.settings.set_keep_chosen_components(val)
 
     def save_parameters(self):
-        dial = SaveDialog(io_functions.save_parameters_dict, False, history=self.settings.get_path_history())
+        dial = CustomSaveDialog(io_functions.save_parameters_dict, False, history=self.settings.get_path_history())
         if not dial.exec_():
             return
         res = dial.get_result()
@@ -669,7 +659,7 @@ class AlgorithmOptions(QWidget):
         self._execute_in_background_init()
 
     def execute_all_action(self):
-        dial = SaveDialog(
+        dial = CustomSaveDialog(
             {SaveROI.get_name(): SaveROI},
             history=self.settings.get_path_history(),
             system_widget=False,
@@ -772,8 +762,8 @@ class ImageInformation(QWidget):
             el.setRange(0, 100000)
             # noinspection PyUnresolvedReferences
             el.valueChanged.connect(self.image_spacing_change)
-        self.units = EnumComboBox(Units)
-        self.units.set_value(units_value)
+        self.units = QEnumComboBox(enum_class=Units)
+        self.units.setCurrentEnum(units_value)
         # noinspection PyUnresolvedReferences
         self.units.currentIndexChanged.connect(self.update_spacing)
 
@@ -800,7 +790,7 @@ class ImageInformation(QWidget):
         self._settings.set("multiple_files_widget", val)
 
     def update_spacing(self, index=None):
-        units_value = self.units.get_value()
+        units_value = self.units.currentEnum()
         if index is not None:
             self._settings.set("units_value", units_value)
         for el, val in zip(self.spacing, self._settings.image_spacing[::-1]):
@@ -896,6 +886,7 @@ class MainWindow(BaseMainWindow):
         view_menu.addAction("Additional output with data").triggered.connect(lambda: self.additional_layers_show(True))
         view_menu.addAction("Napari viewer").triggered.connect(self.napari_viewer_show)
         view_menu.addAction("Toggle Multiple Files").triggered.connect(self.toggle_multiple_files)
+        view_menu.addAction("Toggle console").triggered.connect(self._toggle_console)
         action = view_menu.addAction("Screenshot")
         action.triggered.connect(self.screenshot(self.image_view))
         action.setShortcut(QKeySequence.Print)
@@ -949,7 +940,6 @@ class MainWindow(BaseMainWindow):
             del self.main_menu.measurements_window
         del self.main_menu.segmentation_dialog
         del self.options_panel.algorithm_options.show_parameters_widget
-        self.settings.dump()
         super().closeEvent(event)
 
     @staticmethod
@@ -959,6 +949,7 @@ class MainWindow(BaseMainWindow):
     def set_data(self, data):
         self.main_menu.set_data(data)
 
-    def change_theme(self):
+    # @ensure_main_thread
+    def change_theme(self, event):
         self.image_view.set_theme(self.settings.theme_name)
-        super().change_theme()
+        super().change_theme(event)
