@@ -1,11 +1,13 @@
 import json
 import os
 import os.path
+import re
 import sys
 import warnings
+from argparse import Namespace
 from datetime import datetime
 from pathlib import Path
-from typing import Any, Dict, List, NamedTuple, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Dict, List, NamedTuple, Optional, Tuple, Union
 
 import napari.utils.theme
 import numpy as np
@@ -14,6 +16,7 @@ from napari.utils.theme import template as napari_template
 from qtpy.QtCore import QObject, Signal
 from qtpy.QtWidgets import QMessageBox, QWidget
 
+from PartSeg.common_backend import napari_get_settings
 from PartSeg.common_backend.partially_const_dict import PartiallyConstDict
 from PartSegCore import register
 from PartSegCore.color_image import default_colormap_dict, default_label_dict
@@ -25,31 +28,33 @@ from PartSegCore.roi_info import ROIInfo
 from PartSegCore.segmentation.algorithm_base import ROIExtractionResult
 from PartSegImage import Image
 
+if hasattr(napari.utils.theme, "get_theme"):
+
+    get_theme = napari.utils.theme.get_theme
+
+
+else:  # pragma: no cover
+
+    def get_theme(name: str, as_dict=True) -> Union[dict, Namespace]:
+        theme = napari.utils.theme.palettes[name]
+        theme["canvas"] = "black"
+        if as_dict:
+            return theme
+        return Namespace(**{k: Color(v) if isinstance(v, str) and v.startswith("rgb") else v for k, v in theme.items()})
+
+
 try:
     from napari.qt import get_stylesheet
 except ImportError:  # pragma: no cover
     from napari.resources import get_stylesheet
 
-if hasattr(napari.utils.theme, "get_theme"):
-
-    def get_theme(name: str) -> dict:
-        theme = napari.utils.theme.get_theme(name)
-        if "canvas" in theme and theme["canvas"] != "black":
-            theme["canvas"] = "black"
-            napari.utils.theme.register_theme(name, theme)
-        return theme
-
-
-else:  # pragma: no cover
-
-    def get_theme(name: str) -> dict:
-        theme = napari.utils.theme.palettes[name]
-        theme["canvas"] = "black"
-        return theme
+if TYPE_CHECKING:
+    from napari.settings import NapariSettings
 
 
 DIR_HISTORY = "io.dir_location_history"
 FILE_HISTORY = "io.files_open_history"
+ROI_NOT_FIT = "roi do not fit to image"
 
 
 class ImageSettings(QObject):
@@ -157,7 +162,7 @@ class ImageSettings(QObject):
             else:
                 self._roi_info = val.fit_to_image(self.image)
         except ValueError:  # pragma: no cover
-            raise ValueError("roi do not fit to image")
+            raise ValueError(ROI_NOT_FIT)
         self._additional_layers = {}
         self.roi_changed.emit(self._roi_info)
 
@@ -279,15 +284,27 @@ class ViewSettings(ImageSettings):
     def theme_name(self) -> str:
         return self.get_from_profile("theme", "light")
 
+    @property
+    def theme(self):
+        try:
+            return get_theme(self.theme_name, as_dict=False)
+        except TypeError:
+            theme = get_theme(self.theme_name)
+            return Namespace(
+                **{k: Color(v) if isinstance(v, str) and v.startswith("rgb") else v for k, v in theme.items()}
+            )
+
     @property
     def style_sheet(self):
-        theme = get_theme(self.theme_name)
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", FutureWarning)
+            theme = get_theme(self.theme_name)
         # TODO understand qss overwrite mechanism
         return napari_template("\n".join(register.qss_list) + get_stylesheet() + "\n".join(register.qss_list), **theme)
 
     @theme_name.setter
     def theme_name(self, value: str):
-        if value not in napari.utils.theme.palettes:
+        if value not in napari.utils.theme.available_themes():
             raise ValueError(f"Unsupported theme {value}. Supported one: {self.theme_list()}")
         if value == self.theme_name:
             return
@@ -296,7 +313,10 @@ class ViewSettings(ImageSettings):
 
     @staticmethod
     def theme_list():
-        return list(napari.utils.theme.palettes.keys())
+        try:
+            return napari.utils.theme.available_themes()
+        except:  # noqa: E722  # pylint: disable=W0702
+            return ["light"]
 
     @property
     def chosen_colormap(self):
@@ -425,6 +445,8 @@ class BaseSettings(ViewSettings):
 
     def __init__(self, json_path):
         super().__init__()
+        napari_path = os.path.dirname(json_path) if os.path.basename(json_path) in ["analysis", "mask"] else json_path
+        self.napari_settings: "NapariSettings" = napari_get_settings(napari_path)
         self._current_roi_dict = "default"
         self._roi_dict = ProfileDict()
         self.json_folder_path = json_path
@@ -447,6 +469,22 @@ class BaseSettings(ViewSettings):
         self._points = value if value is not None else None
         self.points_changed.emit()
 
+    @property
+    def theme_name(self) -> str:
+        try:
+            theme = self.napari_settings.appearance.theme
+            if self.get_from_profile("first_start", True):
+                theme = "light"
+                self.napari_settings.appearance.theme = theme
+                self.set_in_profile("first_start", False)
+            return theme
+        except AttributeError:
+            return "light"
+
+    @theme_name.setter
+    def theme_name(self, value: str):
+        self.napari_settings.appearance.theme = value
+
     def set_segmentation_result(self, result: ROIExtractionResult):
         if (
             result.file_path is not None and result.file_path != "" and result.file_path != self.image.file_path
@@ -464,13 +502,12 @@ class BaseSettings(ViewSettings):
         self.last_executed_algorithm = result.parameters.algorithm
         self.set(f"algorithms.{result.parameters.algorithm}", result.parameters.values)
         try:
-            roi = self.image.fit_array_to_image(result.roi)
-            alternative_list = {
-                k: self.image.fit_array_to_image(v) for k, v in result.alternative_representation.items()
-            }
+            roi_info = result.roi_info.fit_to_image(self.image)
         except ValueError:  # pragma: no cover
-            raise ValueError("roi do not fit to image")
-        self._roi_info = ROIInfo(roi, result.roi_annotation, alternative_list)
+            raise ValueError(ROI_NOT_FIT)
+        if result.points is not None:
+            self.points = result.points
+        self._roi_info = roi_info
         self.roi_changed.emit(self._roi_info)
 
     def _load_files_call(self, files_list: List[str]):
@@ -619,6 +656,10 @@ class BaseSettings(ViewSettings):
         :param folder_path: path to directory in which data should be saved.
             If is None then use :py:attr:`.json_folder_path`
         """
+        if self.napari_settings.save is not None:
+            self.napari_settings.save()
+        else:
+            self.napari_settings._save()  # pylint: disable=W0212
         if folder_path is None:
             folder_path = self.json_folder_path
         if not os.path.exists(folder_path):
@@ -629,7 +670,7 @@ class BaseSettings(ViewSettings):
                 dump_string = json.dumps(el.values, cls=self.json_encoder_class, indent=2)
                 with open(os.path.join(folder_path, el.file_name), "w") as ff:
                     ff.write(dump_string)
-            except Exception as e:
+            except Exception as e:  # pylint: disable=W0703
                 errors_list.append((e, os.path.join(folder_path, el.file_name)))
         if errors_list:
             print(errors_list, file=sys.stderr)
@@ -656,7 +697,7 @@ class BaseSettings(ViewSettings):
                     errors_list.append((file_path, data.filter_data()))
                     error = True
                 el.values.update(data)
-            except Exception as e:
+            except Exception as e:  # pylint: disable=W0703
                 error = True
                 errors_list.append((file_path, e))
             finally:
@@ -700,3 +741,12 @@ class TimeAndStackException(Exception):
     Exception which inform that current image has both time
     and stack dat which is not supported
     """
+
+
+class Color:  # pragma: no cover
+    def __init__(self, text):
+        color_re = re.compile(r"rgb\((\d+), (\d+), (\d+)\)")
+        self.colors = tuple(map(int, color_re.match(text).groups()))
+
+    def as_rgb_tuple(self):
+        return self.colors
