@@ -15,9 +15,9 @@ class SaveProperty(typing.NamedTuple):
 
 
 class FormDialog(QDialog):
-    def __init__(self, fields, values=None, image=None):
+    def __init__(self, fields, values=None, image=None, settings=None):
         super().__init__()
-        self.widget = FormWidget(fields)
+        self.widget = FormWidget(fields, settings=settings)
         if values is not None:
             self.widget.set_values(values)
         if image is not None:
@@ -39,16 +39,18 @@ class FormDialog(QDialog):
         return self.widget.set_values(values)
 
 
-class SaveDialog(QFileDialog):
+class CustomSaveDialog(QFileDialog):
     def __init__(
         self,
-        save_register: typing.Dict[str, type(SaveBase)],
+        save_register: typing.Union[typing.Dict[str, type(SaveBase)], type(SaveBase)],
         system_widget=True,
         base_values: typing.Optional[dict] = None,
         parent=None,
         history: typing.Optional[typing.List[str]] = None,
         file_mode=QFileDialog.AnyFile,
     ):
+        if not isinstance(save_register, dict):
+            save_register = {save_register.get_name(): save_register}
         super().__init__(parent)
         self.setFileMode(file_mode)
         self.save_register = {x.get_name_with_suffix(): x for x in save_register.values()}
@@ -143,3 +145,6 @@ class SaveDialog(QFileDialog):
             self.save_register[self.selectedNameFilter()],
             self.values,
         )
+
+
+SaveDialog = CustomSaveDialog
