@@ -48,10 +48,8 @@ class EqualColumnLayout(QLayout):
         return len(self._item_list)
 
     def calc_position(self, rect: QRect):
-        columns = 0
-        for el in self._item_list:
-            if el.widget() and el.widget().isVisible():
-                columns += 1
+        columns = sum(1 for el in self._item_list if el.widget() and el.widget().isVisible())
+
         if columns == 0:
             return
         element_width = rect.width() // columns
@@ -61,3 +59,20 @@ class EqualColumnLayout(QLayout):
                 continue
             el.setGeometry(QRect(x, rect.y(), element_width, rect.height()))
             x += element_width
+
+    def deleteLater(self):
+        for el in self._item_list:
+            if el.widget():
+                el.widget().deleteLater()
+            else:
+                el.layout().deleteLater()
+        self._item_list = []
+        super().deleteLater()
+
+    def __del__(self):
+        for el in self._item_list:
+            if el.widget():
+                el.widget().deleteLater()
+            else:
+                el.layout().deleteLater()
+        self._item_list = []
