@@ -3,9 +3,9 @@ import os
 from pathlib import Path
 from typing import List, Optional, Type
 
-from qtpy.QtCore import Signal
+from qtpy.QtCore import Qt, Signal
 from qtpy.QtGui import QCloseEvent, QDragEnterEvent, QDropEvent, QShowEvent
-from qtpy.QtWidgets import QAction, QApplication, QFileDialog, QMainWindow, QMenu, QMessageBox, QWidget
+from qtpy.QtWidgets import QAction, QApplication, QDockWidget, QFileDialog, QMainWindow, QMenu, QMessageBox, QWidget
 from vispy.color import colormap
 
 from PartSegCore.algorithm_describe_base import Register
@@ -16,10 +16,12 @@ from PartSegImage import Image
 from ..common_backend.base_settings import FILE_HISTORY, BaseSettings, SwapTimeStackException, TimeAndStackException
 from ..common_backend.load_backup import import_config
 from .about_dialog import AboutDialog
-from .custom_save_dialog import SaveDialog
+from .custom_save_dialog import CustomSaveDialog
+from .exception_hooks import load_data_exception_hook
 from .image_adjustment import ImageAdjustmentDialog
 from .napari_image_view import ImageView
 from .napari_viewer_wrap import Viewer
+from .qt_console import QtConsole
 from .show_directory_dialog import DirectoryDialog
 from .waiting_dialog import ExecuteFunctionDialog
 
@@ -76,7 +78,7 @@ class BaseMainMenu(QWidget):
                 # noinspection PyProtectedMember
                 data = dataclasses.replace(data, image=image)
         if data is None:
-            QMessageBox().warning(self, "Data load fail", "Fail with loading data", QMessageBox.Ok)
+            QMessageBox().warning(self, "Data loading failure", "Error during data loading", QMessageBox.Ok)
             return
         self.settings.set_project_info(data)
         if data.file_path:
@@ -117,9 +119,9 @@ class BaseMainWindow(QMainWindow):
         if settings is None:
             if config_folder is None:
                 raise ValueError("wrong config folder")
-            settings: BaseSettings = self.get_setting_class()(config_folder)
             if not os.path.exists(config_folder):
                 import_config()
+            settings: BaseSettings = self.get_setting_class()(config_folder)
             errors = settings.load()
             if errors:
                 errors_message = QMessageBox()
@@ -154,7 +156,20 @@ class BaseMainWindow(QMainWindow):
         self.recent_file_menu = QMenu("Open recent")
         self._refresh_recent(FILE_HISTORY, self.settings.get_last_files())
         self.settings.data_changed.connect(self._refresh_recent)
+        self.settings.napari_settings.appearance.events.theme.connect(self.change_theme)
         self.settings.set_parent(self)
+        self.console = None
+        self.console_dock = QDockWidget("console", self)
+        self.console_dock.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.BottomDockWidgetArea)
+        # self.console_dock.setWidget(self.console)
+        self.console_dock.hide()
+        self.addDockWidget(Qt.BottomDockWidgetArea, self.console_dock)
+
+    def _toggle_console(self):
+        if self.console is None:
+            self.console = QtConsole(self)
+            self.console_dock.setWidget(self.console)
+        self.console_dock.setVisible(not self.console_dock.isVisible())
 
     def _refresh_recent(self, name, value):
         if name != FILE_HISTORY or self._load_dict is None:
@@ -170,7 +185,7 @@ class BaseMainWindow(QMainWindow):
         data = sender.data()
         try:
             method: LoadBase = self._load_dict[data[1]]
-            dial = ExecuteFunctionDialog(method.load, [data[0]])
+            dial = ExecuteFunctionDialog(method.load, [data[0]], exception_hook=load_data_exception_hook)
             if dial.exec():
                 result = dial.get_result()
                 self.main_menu.set_data(result)
@@ -211,10 +226,13 @@ class BaseMainWindow(QMainWindow):
                 self.viewer_list.pop(i)
                 break
 
-    def change_theme(self):
+    # @ensure_main_thread
+    def change_theme(self, event):
+        style_sheet = self.settings.style_sheet
         app = QApplication.instance()
         if app is not None:
-            app.setStyleSheet(self.settings.style_sheet)
+            app.setStyleSheet(style_sheet)
+        self.setStyleSheet(style_sheet)
 
     def showEvent(self, a0: QShowEvent):
         self.show_signal.emit()
@@ -292,12 +310,14 @@ class BaseMainWindow(QMainWindow):
         for el in self.viewer_list:
             el.close()
             del el
+        self.settings.napari_settings.appearance.events.theme.disconnect(self.change_theme)
+        self.settings.dump()
         super().closeEvent(event)
 
     def screenshot(self, viewer: ImageView):
         def _screenshot():
             data = viewer.viewer_widget.screenshot()
-            dial = SaveDialog(
+            dial = CustomSaveDialog(
                 {SaveScreenshot.get_name(): SaveScreenshot},
                 history=self.settings.get_path_history(),
                 system_widget=False,
@@ -317,3 +337,7 @@ class BaseMainWindow(QMainWindow):
     def image_read(self):
         self.setWindowTitle(f"{self.title_base}: {os.path.basename(self.settings.image_path)}")
         self.statusBar().showMessage(self.settings.image_path)
+
+    def deleteLater(self) -> None:
+        self.settings.napari_settings.appearance.events.theme.disconnect(self.change_theme)
+        super().deleteLater()
