@@ -28,6 +28,8 @@ from typing import Any, Callable, Dict, List, Tuple
 
 __author__ = "Grzegorz Bokota"
 
+from PartSegCore.plugins import register_if_need
+
 
 class SubprocessOrder(Enum):
     """
@@ -36,6 +38,7 @@ class SubprocessOrder(Enum):
 
     kill = 1
     wait = 2
+    cancel_job = 3
 
 
 class BatchManager:
@@ -150,6 +153,9 @@ class BatchManager:
                 self.number_off_process += process_diff
             self.join_all()
 
+    def cancel_work(self, global_parameters):
+        del self.calculation_dict[global_parameters.uuid]
+
     def join_all(self):
         logging.debug(f"Join begin {len(self.process_list)} {self.number_off_process}")
         with self.locker:
@@ -202,6 +208,7 @@ class BatchWorker:
         self.order_queue = order_queue
         self.result_queue = result_queue
         self.calculation_dict = calculation_dict
+        self.canceled_tasks = set()
 
     def calculate_task(self, val: Tuple[Any, uuid.UUID]):
         """
@@ -210,11 +217,14 @@ class BatchWorker:
         function and global parameters are obtained from :py:attr:`.calculation_dict`
         """
         data, task_uuid = val
-        global_data, fun = self.calculation_dict[task_uuid]
+        calc = self.calculation_dict.get(task_uuid)
+        if calc is None:
+            self.result_queue.put((task_uuid, (-1, [SubprocessOrder.cancel_job])))
+        global_data, fun = calc
         try:
             res = fun(data, global_data)
             self.result_queue.put((task_uuid, res))
-        except Exception as e:  # pragma: no cover
+        except Exception as e:  # pragma: no cover # pylint: disable=W0703
             traceback.print_exc()
             exc_type, _exc_obj, exc_tb = sys.exc_info()
             f_name = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
@@ -240,11 +250,9 @@ class BatchWorker:
                 except Empty:
                     time.sleep(0.1)
                     continue
-                except MemoryError:  # pragma: no cover
+                except (MemoryError, OSError):  # pragma: no cover
                     pass
-                except OSError:  # pragma: no cover
-                    pass
-                except Exception as ex:  # pragma: no cover
+                except Exception as ex:  # pragma: no cover # pylint: disable=W0703
                     logging.warning(f"Unsupported exception {ex}")
             else:
                 time.sleep(0.1)
@@ -260,5 +268,12 @@ def spawn_worker(task_queue: Queue, order_queue: Queue, result_queue: Queue, cal
     :param result_queue: Queue for calculation result
     :param calculation_dict: dict with global parameters
     """
+    register_if_need()
+    try:
+        from PartSeg.plugins import register_if_need as register
+
+        register()
+    except ImportError:
+        pass
     worker = BatchWorker(task_queue, order_queue, result_queue, calculation_dict)
     worker.run()
