@@ -158,7 +158,9 @@ def get_save_path(op: Save, calculation: "FileCalculation") -> str:
     rel_path = os.path.relpath(calculation.file_path, calculation.base_prefix)
     rel_path = os.path.splitext(rel_path)[0]
     if op.directory:
-        return os.path.join(calculation.result_prefix, rel_path, op.suffix + extension)
+        file_name = os.path.basename(rel_path)
+        base_rel_path = os.path.dirname(rel_path)
+        return os.path.join(calculation.result_prefix, base_rel_path, op.directory, file_name + op.suffix + extension)
     return os.path.join(calculation.result_prefix, rel_path + op.suffix + extension)
 
 
@@ -259,9 +261,7 @@ class MaskFile(MaskMapper, BaseSerializableClass):
             self.parse_map()
         try:
             return self.name_dict[os.path.normpath(file_path)]
-        except KeyError:
-            return ""
-        except AttributeError:
+        except (KeyError, AttributeError):
             return ""
 
     def get_parameters(self):
@@ -411,6 +411,10 @@ class Calculation(BaseCalculation):
         base.uuid = self.uuid
         return base
 
+    @property
+    def measurement(self):
+        return self.calculation_plan.get_measurements()
+
 
 class FileCalculation:
     """
@@ -741,13 +745,9 @@ class CalculationPlan:
         if isinstance(el, Save):
             base = el.short_name
             if el.directory:
-                text = f"Save {base} in directory with name " + el.suffix
-            else:
-                if el.suffix != "":
-                    text = "Save " + base + " with suffix " + el.suffix
-                else:
-                    text = "Save " + base
-            return text
+                return f"Save {base} in directory with name " + el.suffix
+            return "Save " + base + " with suffix " + el.suffix if el.suffix != "" else "Save " + base
+
         if isinstance(el, MaskIntersection):
             if el.name == "":
                 return f"Mask intersection of mask {el.mask1} and {el.mask2}"
