@@ -1,10 +1,13 @@
 from abc import ABC
 from enum import Enum
-from typing import Dict, Optional, Set, Union
+from typing import Any, Dict, List, Optional, Set, Union
 
-from sympy import symbols
+import numpy as np
+from sympy import Symbol, symbols
 
-from ..algorithm_describe_base import AlgorithmDescribeBase, AlgorithmDescribeNotFound
+from PartSegImage.image import Spacing
+
+from ..algorithm_describe_base import AlgorithmDescribeBase, AlgorithmDescribeNotFound, AlgorithmProperty
 from ..channel_class import Channel
 from ..class_generator import BaseSerializableClass, enum_register
 from ..universal_const import Units
@@ -58,7 +61,7 @@ class Leaf(BaseSerializableClass):
         ...
 
     name: str
-    dict: Dict = dict()
+    dict: Dict = {}
     power: float = 1.0
     area: Optional[AreaType] = None
     per_component: Optional[PerComponent] = None
@@ -83,53 +86,52 @@ class Leaf(BaseSerializableClass):
             raise AlgorithmDescribeNotFound(self.name)
         return resp
 
-    def pretty_print(self, measurement_dict: Dict[str, "MeasurementMethodBase"]):  # pragma: no cover
+    def _parameters_string(self, measurement_dict: Dict[str, "MeasurementMethodBase"]) -> str:
+        if len(self.dict) == 0 and self.channel is None:
+            return ""
+        arr = []
+        if self.channel is not None and self.channel >= 0:
+            arr.append(f"channel={self.channel+1}")
+        if self.name in measurement_dict:
+            measurement_method = measurement_dict[self.name]
+            fields_dict = measurement_method.get_fields_dict()
+            for k, v in self.dict.items():
+                arr.append(f"{fields_dict[k].user_name}={v}")
+        else:
+            for k, v in self.dict.items():
+                arr.append(f"{k.replace('_', ' ')}={v}")
+        return "[" + ", ".join(arr) + "]"
+
+    def _plugin_info(self, measurement_dict: Dict[str, "MeasurementMethodBase"]) -> str:
+        if self.name not in measurement_dict:
+            return ""
+        measurement_method = measurement_dict[self.name]
+        if (
+            hasattr(measurement_method, "__module__")
+            and measurement_method.__module__.split(".", 1)[0] != "PartSegCore"
+        ):
+            return f"[{measurement_method.__module__.split('.', 1)[0]}] "
+        return ""
+
+    def pretty_print(self, measurement_dict: Dict[str, "MeasurementMethodBase"]) -> str:  # pragma: no cover
         resp = self.name
         if self.area is not None:
             resp = str(self.area) + " " + resp
-        if self.per_component is not None and self.per_component == PerComponent.Yes:
-            resp += " per component "
-        if self.per_component is not None and self.per_component == PerComponent.Mean:
-            resp += " mean component "
-        if len(self.dict) != 0 or self.channel is not None:
-            resp += "["
-            arr = []
-            if self.channel is not None and self.channel >= 0:
-                arr.append(f"channel={self.channel+1}")
-            if len(self.dict) > 0:
-                try:
-                    measurement_method = measurement_dict[self.name]
-                    fields_dict = measurement_method.get_fields_dict()
-                    for k, v in self.dict.items():
-                        arr.append(f"{fields_dict[k].user_name}={v}")
-                except KeyError:
-                    arr.append("class not found")
-            resp += ", ".join(arr)
-            resp += "]"
+        resp = self._plugin_info(measurement_dict) + resp
+        if self.per_component is not None:
+            if self.per_component == PerComponent.Yes:
+                resp += " per component "
+            elif self.per_component == PerComponent.Mean:
+                resp += " mean component "
+        resp += self._parameters_string(measurement_dict)
         if self.power != 1.0:
             resp += f" to the power {self.power}"
         return resp
 
     def __str__(self):  # pragma: no cover
-        resp = self.name
-        if self.area is not None:
-            resp = str(self.area) + " " + resp
-        if self.per_component is not None and self.per_component == PerComponent.Yes:
-            resp += " per component "
-        if len(self.dict) != 0 or self.channel is not None:
-            resp += "["
-            arr = []
-            if self.channel is not None and self.channel >= 0:
-                arr.append(f"channel={self.channel}")
-            for k, v in self.dict.items():
-                arr.append(f"{k.replace('_', ' ')}={v}")
-            resp += ", ".join(arr)
-            resp += "]"
-        if self.power != 1.0:
-            resp += f" to the power {self.power}"
-        return resp
+        return self.pretty_print({})
 
-    def get_unit(self, ndim):
+    def get_unit(self, ndim) -> Symbol:
         from PartSegCore.analysis import MEASUREMENT_DICT
 
         method = MEASUREMENT_DICT[self.name]
@@ -137,17 +139,17 @@ class Leaf(BaseSerializableClass):
             return method.get_units(ndim) ** self.power
         return method.get_units(ndim)
 
-    def is_per_component(self):
+    def is_per_component(self) -> bool:
         return self.per_component == PerComponent.Yes
 
 
-def replace(self, **kwargs):
+def replace(self, **kwargs) -> Leaf:
     for key in list(kwargs.keys()):
         if key == "power":
             continue
         if not hasattr(self, key):
             raise ValueError(f"Unknown parameter {key}")
-        if getattr(self, key) is not None:
+        if getattr(self, key) is not None and (key != "dict" or self.dict):
             del kwargs[key]
 
     dkt = self.asdict()
@@ -176,7 +178,7 @@ class Node(BaseSerializableClass):
         right_text = "(" + str(self.right) + ")" if isinstance(self.right, Node) else str(self.right)
         return left_text + self.op + right_text
 
-    def pretty_print(self, measurement_dict: Dict[str, "MeasurementMethodBase"]):  # pragma: no cover
+    def pretty_print(self, measurement_dict: Dict[str, "MeasurementMethodBase"]) -> str:  # pragma: no cover
         left_text = (
             "(" + self.left.pretty_print(measurement_dict) + ")"
             if isinstance(self.left, Node)
@@ -189,12 +191,12 @@ class Node(BaseSerializableClass):
         )
         return left_text + self.op + right_text
 
-    def get_unit(self, ndim):
+    def get_unit(self, ndim) -> Symbol:
         if self.op == "/":
             return self.left.get_unit(ndim) / self.right.get_unit(ndim)
         raise ValueError(f"Unknown operator '{self.op}'")
 
-    def is_per_component(self):
+    def is_per_component(self) -> bool:
         return self.left.is_per_component() or self.right.is_per_component()
 
 
@@ -211,7 +213,7 @@ class MeasurementEntry(BaseSerializableClass):
     name: str
     calculation_tree: Union[Node, Leaf]
 
-    def get_unit(self, unit: Units, ndim):
+    def get_unit(self, unit: Units, ndim) -> str:
         return str(self.calculation_tree.get_unit(ndim)).format(str(unit))
 
     def get_channel_num(self, measurement_dict: Dict[str, "MeasurementMethodBase"]) -> Set[Channel]:
@@ -236,33 +238,66 @@ class MeasurementMethodBase(AlgorithmDescribeBase, ABC):
     ]
 
     @classmethod
-    def get_name(cls):
+    def get_name(cls) -> str:
         return str(cls.get_starting_leaf().name)
 
     @classmethod
-    def get_description(cls):
+    def get_description(cls) -> str:
         """Measurement long description"""
+        if isinstance(cls.text_info, str):
+            return ""
         return cls.text_info[1]
 
     @classmethod
-    def is_component(cls):
+    def is_component(cls) -> bool:
         """Return information if Need information about components"""
         return False
 
     @classmethod
-    def get_fields(cls):
+    def get_fields(cls) -> List[Union[str, AlgorithmProperty]]:
         """Additional fields needed by algorithm. like radius of dilation"""
         return []
 
     @staticmethod
-    def calculate_property(**kwargs):
-        """Main function for calculating measurement"""
+    def calculate_property(
+        # image: Image,
+        channel: np.ndarray,
+        roi: np.ndarray,
+        mask: np.ndarray,
+        voxel_size: Spacing,
+        result_scalar: float,
+        roi_alternative: Dict[str, np.ndarray],
+        roi_annotation: Dict[int, Any],
+        **kwargs,
+    ):
+        """
+        Main function for calculating measurement
+
+        :param channel: main channel selected for measurement
+        :param channel_{i}: for channel requested using :py:meth:`get_fields`
+            ``AlgorithmProperty("channel", "Channel", 0, value_type=Channel)``
+        :param area_array: array representing current area returned by :py:meth:`area_type`
+        :param roi: array representing roi
+        :param mask: array representing mask (upper level roi)
+        :param voxel_size: size of single voxel in meters
+        :param result_scalar: scalar to get proper units in result
+        :param roi_alternative: dict with alternative roi representation (for plugin specific mapping)
+        :param roi_annotation: dict with roi annotations (for plugin specific mapping)
+
+        List incomplete.
+        """
         raise NotImplementedError()
 
     @classmethod
     def get_starting_leaf(cls) -> Leaf:
         """This leaf is putted on default list"""
-        return Leaf(cls.text_info[0])
+        return Leaf(cls._display_name())
+
+    @classmethod
+    def _display_name(cls):
+        if isinstance(cls.text_info, str):
+            return cls.text_info
+        return cls.text_info[0]
 
     @classmethod
     def get_units(cls, ndim) -> symbols:
@@ -271,10 +306,14 @@ class MeasurementMethodBase(AlgorithmDescribeBase, ABC):
 
     @classmethod
     def need_channel(cls):
-        """TBA"""
+        """if need image data"""
         return False
 
     @staticmethod
     def area_type(area: AreaType):
         """Map chosen area type to proper area type. Allow to correct Area type."""
         return area
+
+    @staticmethod
+    def need_full_data():
+        return False
