@@ -12,6 +12,7 @@ from typing import (
     MutableMapping,
     NamedTuple,
     Optional,
+    Sequence,
     Set,
     Tuple,
     Union,
@@ -24,15 +25,17 @@ from mahotas.features import haralick
 from scipy.spatial.distance import cdist
 from sympy import symbols
 
+from PartSegCore.segmentation.restartable_segmentation_algorithms import LowerThresholdAlgorithm
 from PartSegImage import Image
 
 from .. import autofit as af
-from ..algorithm_describe_base import AlgorithmProperty, Register
+from ..algorithm_describe_base import AlgorithmProperty, Register, ROIExtractionProfile
 from ..channel_class import Channel
 from ..class_generator import enum_register
 from ..mask_partition_utils import BorderRim, MaskDistanceSplit
 from ..roi_info import ROIInfo
 from ..universal_const import UNIT_SCALE, Units
+from .calculate_pipeline import calculate_segmentation_step
 from .measurement_base import AreaType, Leaf, MeasurementEntry, MeasurementMethodBase, Node, PerComponent
 
 # TODO change image to channel in signature of measurement calculate_property
@@ -40,6 +43,16 @@ from .measurement_base import AreaType, Leaf, MeasurementEntry, MeasurementMetho
 NO_COMPONENT = -1
 
 
+class CorrelationEnum(str, Enum):
+    pearson = "Pearson correlation coefficient"
+    manders = "Mander's overlap coefficient"
+    intensity = "Intensity correlation quotient"
+    spearman = "Spearman rank correlation"
+
+    def __str__(self):
+        return self.value
+
+
 class ProhibitedDivision(Exception):
     pass
 
@@ -193,21 +206,15 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
                 res.append(val)
         return res
 
-    def get_separated(self) -> List[List[MeasurementValueType]]:
-        """Get measurements separated for each component"""
-        has_mask_components, has_segmentation_components = self.get_component_info()
-        if not has_mask_components and not has_segmentation_components:
-            return [list(self._data_dict.values())]
-        if has_mask_components and has_segmentation_components:
-            translation = self.components_info.components_translation
-            component_info = [(x, y) for x in translation.keys() for y in translation[x]]
-        elif has_mask_components:
-            component_info = [(0, x) for x in self.components_info.mask_components]
-        else:
-            component_info = [(x, 0) for x in self.components_info.roi_components]
-        counts = len(component_info)
-        mask_to_pos = {val: i for i, val in enumerate(self.components_info.mask_components)}
-        segmentation_to_pos = {val: i for i, val in enumerate(self.components_info.roi_components)}
+    def _get_component_info(self, has_mask_components, has_segmentation_components):
+        if has_mask_components:
+            if has_segmentation_components:
+                translation = self.components_info.components_translation
+                return [(x, y) for x in translation.keys() for y in translation[x]]
+            return [(0, x) for x in self.components_info.mask_components]
+        return [(x, 0) for x in self.components_info.roi_components]
+
+    def _prepare_res_iterator(self, counts):
         if FILE_NAME_STR in self._data_dict:
             name = self._data_dict[FILE_NAME_STR]
             res = [[name] for _ in range(counts)]
@@ -219,23 +226,34 @@ class MeasurementResult(MutableMapping[str, MeasurementResultType]):
         else:
             res = [[] for _ in range(counts)]
             iterator = iter(self._data_dict.keys())
-        if has_segmentation_components:
-            for i, num in enumerate(component_info):
+        return res, iterator
+
+    def get_separated(self) -> List[List[MeasurementValueType]]:
+        """Get measurements separated for each component"""
+        has_mask_components, has_segmentation_components = self.get_component_info()
+        if not has_mask_components and not has_segmentation_components:
+            return [list(self._data_dict.values())]
+        component_info = self._get_component_info(has_mask_components, has_segmentation_components)
+        res, iterator = self._prepare_res_iterator(len(component_info))
+
+        for i, num in enumerate(component_info):
+            if has_segmentation_components:
                 res[i].append(num[0])
-        if has_mask_components:
-            for i, num in enumerate(component_info):
+            if has_mask_components:
                 res[i].append(num[1])
+
+        mask_to_pos = {val: i for i, val in enumerate(self.components_info.mask_components)}
+        segmentation_to_pos = {val: i for i, val in enumerate(self.components_info.roi_components)}
+
         for el in iterator:
             per_comp, area_type = self._type_dict[el]
             val = self._data_dict[el]
-            if per_comp != PerComponent.Yes:
-                for i in range(counts):
+            for i, (seg, mask) in enumerate(component_info):
+                if per_comp != PerComponent.Yes:
                     res[i].append(val)
-            elif area_type == AreaType.ROI:
-                for i, (seg, _mask) in enumerate(component_info):
+                elif area_type == AreaType.ROI:
                     res[i].append(val[segmentation_to_pos[seg]])
-            else:
-                for i, (_seg, mask) in enumerate(component_info):
+                else:
                     res[i].append(val[mask_to_pos[mask]])
         return res
 
@@ -324,11 +342,8 @@ class MeasurementProfile:
         return self._is_component_measurement(node.left) or self._is_component_measurement(node.right)
 
     @staticmethod
-    def _calculate_leaf_value(
-        node: Union[Node, Leaf], segmentation_mask_map: ComponentsInfo, help_dict: dict, kwargs: dict
-    ) -> Union[float, np.ndarray]:
-        method: MeasurementMethodBase = MEASUREMENT_DICT[node.name]
-        area_type = method.area_type(node.area)
+    def _prepare_leaf_kw(node, kwargs, method, area_type):
+
         area_type_dict = {
             AreaType.Mask: "mask",
             AreaType.Mask_without_ROI: "mask_without_segmentation",
@@ -342,30 +357,57 @@ class MeasurementProfile:
             kw["channel_num"] = node.channel
         else:
             kw["channel_num"] = -1
-        kw["help_dict"] = help_dict
+
         kw["_area"] = node.area
         kw["_per_component"] = node.per_component
         kw["_cache"] = True  # TODO remove cache argument
         kw["area_array"] = kw[area_type_dict[area_type]]
         kw["_component_num"] = NO_COMPONENT
+        return kw
+
+    def _clip_arrays(self, kw, node: Leaf, method: MeasurementMethodBase, component_index: int):
+        bounds = tuple(kw["bounds_info"][component_index].get_slices(margin=1))
+        if node.area != AreaType.ROI or method.need_full_data():
+            bounds = tuple(slice(None, None) for _ in bounds)
+        kw2 = kw.copy()
+        kw2["_component_num"] = component_index
+
+        kw2["area_array"] = kw["area_array"][bounds] == component_index
+        im_bounds = list(bounds)
+        image: Image = kw["image"]
+        im_bounds.insert(image.time_pos if image.time_pos < image.channel_pos else image.time_pos - 1, slice(None))
+        im_mask = image.mask[tuple(im_bounds)] if image.mask is not None else None
+        im_bounds.insert(image.channel_pos, slice(None))
+        kw2["image"] = kw["image"].substitute(data=image.get_data()[tuple(im_bounds)], mask=im_mask)
+        for name in ["channel", "segmentation", "roi", "mask"] + [f"channel_{num}" for num in self.get_channels_num()]:
+            if kw[name] is not None:
+                kw2[name] = kw[name][bounds]
+        kw2["roi_alternative"] = kw2["roi_alternative"].copy()
+        for name, array in kw2["roi_alternative"].items():
+            kw2["roi_alternative"][name] = array[bounds]
+        return kw2
+
+    def _calculate_leaf_value(
+        self, node: Union[Node, Leaf], segmentation_mask_map: ComponentsInfo, kwargs: dict
+    ) -> Union[float, np.ndarray]:
+        method: MeasurementMethodBase = MEASUREMENT_DICT[node.name]
+        kw = self._prepare_leaf_kw(node, kwargs, method, method.area_type(node.area))
+
         if node.per_component == PerComponent.No:
-            val = method.calculate_property(**kw)
+            return method.calculate_property(**kw)
+        # TODO use cache for per component calculate
+        # kw["_cache"] = False
+        val = []
+        if method.area_type(node.area) == AreaType.ROI:
+            components = segmentation_mask_map.roi_components
         else:
-            # TODO use cache for per component calculate
-            # kw["_cache"] = False
-            val = []
-            area_array = kw["area_array"]
-            if area_type == AreaType.ROI:
-                components = segmentation_mask_map.roi_components
-            else:
-                components = segmentation_mask_map.mask_components
-            for i in components:
-                kw["area_array"] = area_array == i
-                kw["_component_num"] = i
-                val.append(method.calculate_property(**kw))
-            val = np.array(val)
-            if node.per_component == PerComponent.Mean:
-                val = np.mean(val) if val.size else 0
+            components = segmentation_mask_map.mask_components
+        for i in components:
+            kw2 = self._clip_arrays(kw, node, method, i)
+            val.append(method.calculate_property(**kw2))
+        val = np.array(val)
+        if node.per_component == PerComponent.Mean:
+            val = np.mean(val) if val.size else 0
         return val
 
     def _calculate_leaf(
@@ -378,7 +420,8 @@ class MeasurementProfile:
         if hash_str in help_dict:
             val = help_dict[hash_str]
         else:
-            val = self._calculate_leaf_value(node, segmentation_mask_map, help_dict, kwargs)
+            kwargs["help_dict"] = help_dict
+            val = self._calculate_leaf_value(node, segmentation_mask_map, kwargs)
             help_dict[hash_str] = val
         unit: symbols = method.get_units(3) if kwargs["image"].is_stack else method.get_units(2)
         if node.power != 1:
@@ -388,37 +431,35 @@ class MeasurementProfile:
     def _calculate_node(
         self, node: Node, segmentation_mask_map: ComponentsInfo, help_dict: dict, kwargs: dict
     ) -> Tuple[Union[float, np.ndarray], symbols, AreaType]:
-        left_res, left_unit, left_area = self.calculate_tree(node.left, segmentation_mask_map, help_dict, kwargs)
-        right_res, right_unit, right_area = self.calculate_tree(node.right, segmentation_mask_map, help_dict, kwargs)
         if node.op != "/":
             raise ValueError(f"Wrong measurement: {node}")
-        if isinstance(left_res, np.ndarray) and isinstance(right_res, np.ndarray) and left_area != right_area:
-            area_set = {left_area, right_area}
-            if area_set == {AreaType.ROI, AreaType.Mask_without_ROI}:  # pragma: no cover
-                raise ProhibitedDivision("This division is prohibited")
-            if area_set == {AreaType.ROI, AreaType.Mask}:
-                res = []
-                reverse = False
-                if left_area == AreaType.Mask:
-                    left_res, right_res = right_res, left_res
-                    reverse = True
-                for val, num in zip(left_res, segmentation_mask_map.roi_components):
-                    div_vals = segmentation_mask_map.components_translation[num]
-                    if len(div_vals) != 1:  # pragma: no cover
-                        raise ProhibitedDivision("Cannot calculate when object do not belongs to one mask area")
-                    if left_area == AreaType.ROI:
-                        res.append(val / right_res[div_vals[0] - 1])
-                    else:
-                        res.append(right_res[div_vals[0] - 1] / val)
-                res = np.array(res)
-                if reverse:
-                    res = 1 / res
-                    # TODO check this area type
-                return res, left_unit / right_unit, AreaType.ROI
-            # TODO check this
-            left_area = AreaType.Mask_without_ROI
+        left_res, left_unit, left_area = self.calculate_tree(node.left, segmentation_mask_map, help_dict, kwargs)
+        right_res, right_unit, right_area = self.calculate_tree(node.right, segmentation_mask_map, help_dict, kwargs)
+        if not (isinstance(left_res, np.ndarray) and isinstance(right_res, np.ndarray) and left_area != right_area):
+            return left_res / right_res, left_unit / right_unit, left_area
+        area_set = {left_area, right_area}
+        if area_set == {AreaType.ROI, AreaType.Mask_without_ROI}:  # pragma: no cover
+            raise ProhibitedDivision("This division is prohibited")
+        if area_set == {AreaType.Mask, AreaType.Mask_without_ROI}:
+            return left_res / right_res, left_unit / right_unit, AreaType.Mask_without_ROI
 
-        return left_res / right_res, left_unit / right_unit, left_area
+        # if area_set == {AreaType.ROI, AreaType.Mask}:
+        res = []
+        if left_area == AreaType.Mask:
+            roi_res, mask_res = right_res, left_res
+        else:
+            roi_res, mask_res = left_res, right_res
+        for val, num in zip(roi_res, segmentation_mask_map.roi_components):
+            mask_components = segmentation_mask_map.components_translation[num]
+            if len(mask_components) != 1:  # pragma: no cover
+                raise ProhibitedDivision("Cannot calculate when object do not belongs to one mask area")
+            if left_area == AreaType.ROI:
+                res.append(val / mask_res[mask_components[0] - 1])
+            else:
+                res.append(mask_res[mask_components[0] - 1] / val)
+        res = np.array(res)
+        return res, left_unit / right_unit, AreaType.ROI
+        # TODO check this
 
     def calculate_tree(
         self, node: Union[Node, Leaf], segmentation_mask_map: ComponentsInfo, help_dict: dict, kwargs: dict
@@ -554,22 +595,27 @@ class MeasurementProfile:
         channel = image.get_channel(channel_num).astype(float)
         cache_dict = {}
         result_scalar = UNIT_SCALE[result_units.value]
+        if isinstance(roi, np.ndarray):
+            roi = ROIInfo(roi).fit_to_image(image)
         roi_alternative = {}
-        if isinstance(roi, ROIInfo):
-            for name, array in roi.alternative.items():
-                roi_alternative[name] = get_time(array)
+        for name, array in roi.alternative.items():
+            roi_alternative[name] = get_time(array)
         kw = {
             "image": image,
             "channel": get_time(channel),
-            "segmentation": get_time(roi if isinstance(roi, np.ndarray) else roi.roi),
+            "segmentation": get_time(roi.roi),
+            "roi": get_time(roi.roi),
+            "bounds_info": {
+                k: v.del_dim(image.time_pos) if len(v.lower) == 4 else v for k, v in roi.bound_info.items()
+            },
             "mask": get_time(image.mask),
             "voxel_size": image.spacing,
             "result_scalar": result_scalar,
             "roi_alternative": roi_alternative,
-            "roi_annotation": roi.annotations if isinstance(roi, ROIInfo) else {},
+            "roi_annotation": roi.annotations,
         }
         for num in self.get_channels_num():
-            kw["channel_{num}"] = get_time(image.get_channel(num))
+            kw[f"channel_{num}"] = get_time(image.get_channel(num))
         if any(self._need_mask_without_segmentation(el.calculation_tree) for el in self.chosen_fields):
             mm = kw["mask"].copy()
             mm[kw["segmentation"] > 0] = 0
@@ -802,7 +848,7 @@ class PixelBrightnessSum(MeasurementMethodBase):
             if area_array.size == channel.size:
                 channel = channel.reshape(area_array.shape)
             else:  # pragma: no cover
-                raise ValueError("channel and mask do not fit each other")
+                raise ValueError(f"channel ({channel.shape}) and mask ({area_array.shape}) do not fit each other")
         if np.any(area_array):
             return np.sum(channel[area_array > 0])
         return 0
@@ -825,7 +871,7 @@ class ComponentsNumber(MeasurementMethodBase):
 
     @classmethod
     def get_starting_leaf(cls):
-        return Leaf(cls.text_info[0], per_component=PerComponent.No)
+        return super().get_starting_leaf().replace_(per_component=PerComponent.No)
 
     @classmethod
     def get_units(cls, ndim):
@@ -836,9 +882,9 @@ class MaximumPixelBrightness(MeasurementMethodBase):
     text_info = "Maximum pixel brightness", "Calculate maximum pixel brightness for current area"
 
     @staticmethod
-    def calculate_property(area_array, channel, **_):
+    def calculate_property(area_array, channel, **_):  # pylint: disable=W0221
         if area_array.shape != channel.shape:  # pragma: no cover
-            raise ValueError("channel and mask do not fit each other")
+            raise ValueError(f"channel ({channel.shape}) and mask ({area_array.shape}) do not fit each other")
         if np.any(area_array):
             return np.max(channel[area_array > 0])
         return 0
@@ -856,7 +902,7 @@ class MinimumPixelBrightness(MeasurementMethodBase):
     text_info = "Minimum pixel brightness", "Calculate minimum pixel brightness for current area"
 
     @staticmethod
-    def calculate_property(area_array, channel, **_):
+    def calculate_property(area_array, channel, **_):  # pylint: disable=W0221
         if area_array.shape != channel.shape:  # pragma: no cover
             raise ValueError("channel and mask do not fit each other")
         if np.any(area_array):
@@ -963,7 +1009,7 @@ class FirstPrincipalAxisLength(MeasurementMethodBase):
     text_info = "First principal axis length", "Length of first principal axis"
 
     @staticmethod
-    def calculate_property(**kwargs):
+    def calculate_property(**kwargs):  # pylint: disable=W0221
         return get_main_axis_length(0, **kwargs)
 
     @classmethod
@@ -979,7 +1025,7 @@ class SecondPrincipalAxisLength(MeasurementMethodBase):
     text_info = "Second principal axis length", "Length of second principal axis"
 
     @staticmethod
-    def calculate_property(**kwargs):
+    def calculate_property(**kwargs):  # pylint: disable=W0221
         return get_main_axis_length(1, **kwargs)
 
     @classmethod
@@ -995,7 +1041,7 @@ class ThirdPrincipalAxisLength(MeasurementMethodBase):
     text_info = "Third principal axis length", "Length of third principal axis"
 
     @staticmethod
-    def calculate_property(**kwargs):
+    def calculate_property(**kwargs):  # pylint: disable=W0221
         return get_main_axis_length(2, **kwargs)
 
     @classmethod
@@ -1011,7 +1057,7 @@ class Compactness(MeasurementMethodBase):
     text_info = "Compactness", "Calculate compactness off segmentation (Surface^1.5/volume)"
 
     @staticmethod
-    def calculate_property(**kwargs):
+    def calculate_property(**kwargs):  # pylint: disable=W0221
         cache = kwargs["_cache"] if "_cache" in kwargs else False
         cache = cache and "help_dict" in kwargs
         cache = cache and "_area" in kwargs
@@ -1050,7 +1096,7 @@ class Sphericity(MeasurementMethodBase):
     text_info = "Sphericity", "volume/(4/3 * π * radius **3) for 3d data and volume/(π * radius **2) for 2d data"
 
     @staticmethod
-    def calculate_property(**kwargs):
+    def calculate_property(**kwargs):  # pylint: disable=W0221
         if all(key in kwargs for key in ["help_dict", "_area", "_per_component"]) and (
             "_cache" not in kwargs or kwargs["_cache"]
         ):
@@ -1106,7 +1152,7 @@ class RimVolume(MeasurementMethodBase):
 
     @classmethod
     def get_starting_leaf(cls):
-        return Leaf(name=cls.text_info[0], area=AreaType.Mask)
+        return super().get_starting_leaf().replace_(area=AreaType.Mask)
 
     @staticmethod
     def calculate_property(area_array, voxel_size, result_scalar, **kwargs):  # pylint: disable=W0221
@@ -1137,7 +1183,7 @@ class RimPixelBrightnessSum(MeasurementMethodBase):
 
     @classmethod
     def get_starting_leaf(cls):
-        return Leaf(name=cls.text_info[0], area=AreaType.Mask)
+        return super().get_starting_leaf().replace_(area=AreaType.Mask)
 
     @staticmethod
     def calculate_property(channel, area_array, **kwargs):  # pylint: disable=W0221
@@ -1183,14 +1229,14 @@ except NameError:
     enum_register.register_class(DistancePoint)
 
 
-class DistanceMaskSegmentation(MeasurementMethodBase):
-    text_info = "segmentation distance", "Calculate distance between segmentation and mask"
+class DistanceMaskROI(MeasurementMethodBase):
+    text_info = "ROI distance", "Calculate distance between ROI and mask"
 
     @classmethod
     def get_fields(cls):
         return [
             AlgorithmProperty("distance_from_mask", "Distance from mask", DistancePoint.Border),
-            AlgorithmProperty("distance_to_segmentation", "Distance to segmentation", DistancePoint.Border),
+            AlgorithmProperty("distance_to_segmentation", "Distance to ROI", DistancePoint.Border),
         ]
 
     @staticmethod
@@ -1239,7 +1285,7 @@ class DistanceMaskSegmentation(MeasurementMethodBase):
 
     @classmethod
     def get_starting_leaf(cls):
-        return Leaf(name=cls.text_info[0], area=AreaType.Mask)
+        return super().get_starting_leaf().replace_(area=AreaType.Mask)
 
     @classmethod
     def get_units(cls, ndim):
@@ -1254,6 +1300,156 @@ class DistanceMaskSegmentation(MeasurementMethodBase):
         return AreaType.ROI
 
 
+class DistanceROIROI(DistanceMaskROI):
+    text_info = "to new ROI distance", "Calculate distance between ROI and new ROI"
+
+    @classmethod
+    def get_starting_leaf(cls):
+        return Leaf(name=cls.text_info[0], area=AreaType.ROI)
+
+    @classmethod
+    def get_fields(cls):
+        return [
+            AlgorithmProperty(
+                "profile",
+                "ROI extraction profile",
+                ROIExtractionProfile(
+                    "default", LowerThresholdAlgorithm.get_name(), LowerThresholdAlgorithm.get_default_values()
+                ),
+                value_type=ROIExtractionProfile,
+            ),
+            AlgorithmProperty("distance_from_new_roi", "Distance new ROI", DistancePoint.Border),
+            AlgorithmProperty("distance_to_roi", "Distance to ROI", DistancePoint.Border),
+        ]
+
+    # noinspection PyMethodOverriding
+    @classmethod
+    def calculate_property(
+        cls,
+        channel: np.ndarray,
+        image: Image,
+        area_array: np.ndarray,
+        profile: ROIExtractionProfile,
+        mask: Optional[np.ndarray],
+        voxel_size: Sequence[float],
+        result_scalar: float,
+        distance_from_new_roi: DistancePoint,
+        distance_to_roi: DistancePoint,
+        **kwargs,
+    ):  # pylint: disable=W0221
+        if len(channel.shape) == 4:
+            if channel.shape[0] != 1:
+                raise ValueError("This measurements do not support time data")
+            channel = channel[0]
+        try:
+            hash_name = hash_fun_call_name(
+                calculate_segmentation_step,
+                profile,
+                kwargs["_area"],
+                kwargs["_per_component"],
+                Channel(-1),
+                kwargs["_component_num"],
+            )
+            if hash_name in kwargs["help_dict"]:
+                result = kwargs["help_dict"][hash_name]
+            else:
+                result, _ = calculate_segmentation_step(profile, image, mask)
+                kwargs["help_dict"][hash_name] = result
+        except KeyError:
+            result, _ = calculate_segmentation_step(profile, image, mask)
+
+        if np.any(result.roi[area_array > 0]):
+            return 0
+
+        return super().calculate_property(
+            channel,
+            area_array,
+            result.roi,
+            tuple(voxel_size),
+            result_scalar,
+            distance_from_mask=distance_from_new_roi,
+            distance_to_segmentation=distance_to_roi,
+        )
+
+    @staticmethod
+    def need_full_data():
+        return True
+
+
+class ROINeighbourhoodROI(DistanceMaskROI):
+    text_info = "Neighbourhood new ROI presence", "Count how many of new roi are present in neighbourhood of new ROI"
+
+    @classmethod
+    def get_starting_leaf(cls):
+        return Leaf(name=cls.text_info[0], area=AreaType.ROI)
+
+    @classmethod
+    def get_fields(cls):
+        return [
+            AlgorithmProperty(
+                "profile",
+                "ROI extraction profile",
+                ROIExtractionProfile(
+                    "default", LowerThresholdAlgorithm.get_name(), LowerThresholdAlgorithm.get_default_values()
+                ),
+                value_type=ROIExtractionProfile,
+            ),
+            AlgorithmProperty("distance", "Distance", 500.0, options_range=(0, 10000), value_type=float),
+            AlgorithmProperty("units", "Units", Units.nm, value_type=Units),
+        ]
+
+    # noinspection PyMethodOverriding
+    @classmethod
+    def calculate_property(
+        cls,
+        image: Image,
+        area_array: np.ndarray,
+        profile: ROIExtractionProfile,
+        mask: Optional[np.ndarray],
+        voxel_size,
+        distance: float,
+        units: Units,
+        **kwargs,
+    ):  # pylint: disable=W0221
+        try:
+            hash_name = hash_fun_call_name(
+                calculate_segmentation_step,
+                profile,
+                kwargs["_area"],
+                kwargs["_per_component"],
+                Channel(-1),
+                kwargs["_component_num"],
+            )
+            if hash_name in kwargs["help_dict"]:
+                result = kwargs["help_dict"][hash_name]
+            else:
+                result, _ = calculate_segmentation_step(profile, image, mask)
+                kwargs["help_dict"][hash_name] = result
+        except KeyError:
+            result, _ = calculate_segmentation_step(profile, image, mask)
+        area_array = image.fit_array_to_image(area_array)
+        units_scalar = UNIT_SCALE[units.value]
+        final_radius = [int((distance / units_scalar) / x) for x in reversed(voxel_size)]
+
+        dilated = SimpleITK.GetArrayFromImage(
+            SimpleITK.BinaryDilate(
+                SimpleITK.GetImageFromArray((area_array > 0).astype(np.uint8).squeeze()), final_radius
+            )
+        )
+        dilated = dilated.reshape(area_array.shape)
+        roi = image.fit_array_to_image(result.roi)
+
+        components = set(np.unique(roi[dilated > 0]))
+        if 0 in components:
+            components.remove(0)
+
+        return len(components)
+
+    @staticmethod
+    def need_full_data():
+        return True
+
+
 class SplitOnPartVolume(MeasurementMethodBase):
     text_info = (
         "distance splitting volume",
@@ -1278,7 +1474,7 @@ class SplitOnPartVolume(MeasurementMethodBase):
 
     @classmethod
     def get_starting_leaf(cls):
-        return Leaf(name=cls.text_info[0], area=AreaType.Mask)
+        return super().get_starting_leaf().replace_(area=AreaType.Mask)
 
     @staticmethod
     def area_type(area: AreaType):
@@ -1311,7 +1507,7 @@ class SplitOnPartPixelBrightnessSum(MeasurementMethodBase):
 
     @classmethod
     def get_starting_leaf(cls):
-        return Leaf(name=cls.text_info[0], area=AreaType.Mask)
+        return super().get_starting_leaf().replace_(area=AreaType.Mask)
 
     @staticmethod
     def area_type(area: AreaType):
@@ -1380,6 +1576,106 @@ class Haralick(MeasurementMethodBase):
         return haralick(data, distance=distance, ignore_zeros=True, return_mean=True)
 
 
+class ComponentBoundingBox(MeasurementMethodBase):
+    text_info = "Component Bounding Box", "bounding box as string"
+
+    @classmethod
+    def get_units(cls, ndim):
+        return "str"
+
+    @staticmethod
+    def calculate_property(bounds_info, _component_num, **kwargs):  # pylint: disable=W0221
+        return str(bounds_info[_component_num])
+
+    @classmethod
+    def get_starting_leaf(cls):
+        return super().get_starting_leaf().replace_(area=AreaType.ROI, per_component=PerComponent.Yes)
+
+
+class GetROIAnnotationType(MeasurementMethodBase):
+    text_info = "annotation by name", "Get roi annotation by name"
+
+    @classmethod
+    def get_fields(cls):
+        return [AlgorithmProperty("name", "Name", "")]
+
+    @classmethod
+    def get_starting_leaf(cls):
+        return Leaf(name=cls.text_info[0], area=AreaType.ROI, per_component=PerComponent.Yes)
+
+    @staticmethod
+    def calculate_property(roi_annotation, name, _component_num, **kwargs):  # pylint: disable=W0221
+        return str(roi_annotation.get(_component_num, {}).get(name, ""))
+
+    @classmethod
+    def get_units(cls, ndim):
+        return "str"
+
+
+class ColocalizationMeasurement(MeasurementMethodBase):
+    text_info = "Colocalization", "Measurement of colocalization of two channels."
+
+    @classmethod
+    def get_fields(cls):
+        return [
+            AlgorithmProperty("channel_fst", "Channel 1", 0, value_type=Channel),
+            AlgorithmProperty("channel_scd", "Channel 2", 1, value_type=Channel),
+            AlgorithmProperty(
+                "colocalization",
+                "Colocalization",
+                CorrelationEnum.pearson,
+            ),
+            AlgorithmProperty(
+                "randomize", "Randomize channel", False, help_text="If randomize orders of pixels in one channel"
+            ),
+            AlgorithmProperty(
+                "randomize_repeat", "Randomize num", 10, help_text="Number of repetitions for mean_calculate"
+            ),
+        ]
+
+    @staticmethod
+    def _calculate_masked(data_1, data_2, colocalization):
+        if colocalization == CorrelationEnum.spearman:
+            data_1 = data_1.argsort().argsort().astype(float)
+            data_2 = data_2.argsort().argsort().astype(float)
+            colocalization = CorrelationEnum.pearson
+        if colocalization == CorrelationEnum.pearson:
+            data_1_mean = np.mean(data_1)
+            data_2_mean = np.mean(data_2)
+            nominator = np.sum((data_1 - data_1_mean) * (data_2 - data_2_mean))
+            numerator = np.sqrt(np.sum((data_1 - data_1_mean) ** 2) * np.sum((data_2 - data_2_mean) ** 2))
+            return nominator / numerator
+        if colocalization == CorrelationEnum.manders:
+            nominator = np.sum(data_1 * data_2)
+            numerator = np.sqrt(np.sum(data_1 ** 2) * np.sum(data_2 ** 2))
+            return nominator / numerator
+        if colocalization == CorrelationEnum.intensity:
+            data_1_mean = np.mean(data_1)
+            data_2_mean = np.mean(data_2)
+            return np.sum((data_1 > data_1_mean) == (data_2 > data_2_mean)) / data_1.size - 0.5
+
+        raise RuntimeError(f"Not supported colocalization method {colocalization}")  # pragma: no cover
+
+    @classmethod
+    def calculate_property(
+        cls, area_array, colocalization, randomize=False, randomize_repeat=10, channel_fst=0, channel_scd=1, **kwargs
+    ):  # pylint: disable=W0221
+        mask_binary = area_array > 0
+        data_1 = kwargs[f"channel_{channel_fst}"][mask_binary].astype(float)
+        data_2 = kwargs[f"channel_{channel_scd}"][mask_binary].astype(float)
+        if not randomize:
+            return cls._calculate_masked(data_1, data_2, colocalization)
+        res_list = []
+        for _ in range(randomize_repeat):
+            rand_data2 = np.random.permutation(data_2)
+            res_list.append(cls._calculate_masked(data_1, rand_data2, colocalization))
+        return np.mean(res_list)
+
+    @classmethod
+    def get_units(cls, ndim) -> symbols:
+        return 1
+
+
 def pixel_volume(spacing, result_scalar):
     return reduce((lambda x, y: x * y), [x * result_scalar for x in spacing])
 
@@ -1424,12 +1720,15 @@ MEASUREMENT_DICT = Register(
     Volume,
     Diameter,
     PixelBrightnessSum,
+    ComponentBoundingBox,
+    GetROIAnnotationType,
     ComponentsNumber,
     MaximumPixelBrightness,
     MinimumPixelBrightness,
     MeanPixelBrightness,
     MedianPixelBrightness,
     StandardDeviationOfPixelBrightness,
+    ColocalizationMeasurement,
     Moment,
     FirstPrincipalAxisLength,
     SecondPrincipalAxisLength,
@@ -1439,7 +1738,9 @@ MEASUREMENT_DICT = Register(
     Surface,
     RimVolume,
     RimPixelBrightnessSum,
-    DistanceMaskSegmentation,
+    ROINeighbourhoodROI,
+    DistanceMaskROI,
+    DistanceROIROI,
     SplitOnPartVolume,
     SplitOnPartPixelBrightnessSum,
     Voxels,
