@@ -62,7 +62,7 @@ def save_project(
             segmentation_tar = get_tarinfo("mask.tif", segmentation_buff)
             tar.addfile(segmentation_tar, fileobj=segmentation_buff)
         image_buff = BytesIO()
-        ImageWriter.save(image, image_buff)
+        ImageWriter.save(image, image_buff, compression=None)
         tar_image = get_tarinfo("image.tif", image_buff)
         tar.addfile(tarinfo=tar_image, fileobj=image_buff)
         para_str = json.dumps(algorithm_parameters, cls=PartEncoder)
@@ -363,6 +363,38 @@ class SaveAsNumpy(SaveBase):
         np.save(save_location, data)
 
 
+class SaveProfilesToJSON(SaveBase):
+    @classmethod
+    def get_short_name(cls):
+        return "json"
+
+    @classmethod
+    def save(
+        cls,
+        save_location: typing.Union[str, BytesIO, Path],
+        project_info,
+        parameters: dict,
+        range_changed=None,
+        step_changed=None,
+    ):
+        with open(save_location, "w") as ff:
+            json.dump(project_info, ff, cls=PartEncoder, indent=2)
+
+    @classmethod
+    def get_name(cls) -> str:
+        return "Segment profile (*.json)"
+
+    @classmethod
+    def get_fields(cls) -> typing.List[typing.Union[AlgorithmProperty, str]]:
+        return []
+
+
+class SavePipelineToJSON(SaveProject):
+    @classmethod
+    def get_name(cls) -> str:
+        return "Segment pipeline (*.json)"
+
+
 save_dict = Register(
     SaveProject,
     SaveCmap,
