@@ -1,6 +1,7 @@
 import dataclasses
 import json
 import os
+import sys
 import tarfile
 import typing
 from collections import defaultdict
@@ -12,7 +13,7 @@ import numpy as np
 import tifffile
 from napari.plugins._builtins import napari_write_points
 
-from PartSegImage import GenericImageReader, Image, ImageWriter, TiffImageReader
+from PartSegImage import BaseImageWriter, GenericImageReader, Image, IMAGEJImageWriter, ImageWriter, TiffImageReader
 from PartSegImage.image import FRAME_THICKNESS, reduce_array
 
 from ..algorithm_describe_base import AlgorithmProperty, Register, ROIExtractionProfile
@@ -36,6 +37,9 @@ from ..json_hooks import ProfileEncoder
 from ..project_info import AdditionalLayerDescription, HistoryElement, ProjectInfoBase
 from ..roi_info import ROIInfo
 
+if sys.version_info[:3] == (3, 9, 7):
+    ProjectInfoBase = object  # noqa: F811
+
 
 @dataclasses.dataclass(frozen=True)
 class MaskProjectTuple(ProjectInfoBase):
@@ -75,7 +79,7 @@ class MaskProjectTuple(ProjectInfoBase):
         return MaskProjectTuple(self.file_path, self.image.substitute(mask=None))
 
     def is_raw(self):
-        return self.roi is None
+        return self.roi_info.roi is None
 
     def is_masked(self):
         return self.mask is not None
@@ -109,7 +113,7 @@ def save_stack_segmentation(
             segmentation_info.roi_info.roi, spacing, axes_order=Image.axis_order.replace("C", "")
         )
         try:
-            ImageWriter.save(segmentation_image, segmentation_buff)
+            ImageWriter.save(segmentation_image, segmentation_buff, compression=None)
         except ValueError:
             segmentation_buff.seek(0)
             tifffile.imwrite(segmentation_buff, segmentation_info.roi_info.roi)
@@ -520,29 +524,33 @@ def save_components(
     components: list,
     dir_path: str,
     roi_info: ROIInfo,
+    parameters=None,
     points: typing.Optional[np.ndarray] = None,
     range_changed=None,
     step_changed=None,
+    writer_class: typing.Type[BaseImageWriter] = ImageWriter,
 ):
     if range_changed is None:
         range_changed = empty_fun
     if step_changed is None:
         step_changed = empty_fun
 
+    if parameters is None:
+        parameters = SaveComponents.get_default_values()
+
     roi_info = roi_info.fit_to_image(image)
     os.makedirs(dir_path, exist_ok=True)
 
     file_name = os.path.splitext(os.path.basename(image.file_path))[0]
-    points_casted = None
     important_axis = "XY" if image.is_2d else "XYZ"
     index_to_frame_points = image.calc_index_to_frame(image.axis_order, important_axis)
-    if points is not None:
-        points_casted = points.astype(np.uint16)
-
+    points_casted = points.astype(np.uint16) if points is not None else None
     range_changed(0, 2 * len(components))
     for i in components:
         components_mark = np.array(roi_info.roi == i)
-        im = image.cut_image(components_mark, replace_mask=True)
+        im = image.cut_image(
+            components_mark, replace_mask=True, frame=parameters["frame"], zero_out_cut_area=parameters["mask_data"]
+        )
         if points is not None and points_casted is not None:
             points_mask = components_mark[tuple(points_casted.T)]
             filtered_points = points[points_mask]
@@ -555,9 +563,9 @@ def save_components(
             )
 
         # print(f"[run] {im}")
-        ImageWriter.save(im, os.path.join(dir_path, f"{file_name}_component{i}.tif"))
+        writer_class.save(im, os.path.join(dir_path, f"{file_name}_component{i}.tif"))
         step_changed(2 * i + 1)
-        ImageWriter.save_mask(im, os.path.join(dir_path, f"{file_name}_component{i}_mask.tif"))
+        writer_class.save_mask(im, os.path.join(dir_path, f"{file_name}_component{i}_mask.tif"))
         step_changed(2 * i + 2)
 
 
@@ -584,6 +592,7 @@ class SaveComponents(SaveBase):
             project_info.selected_components,
             save_location,
             project_info.roi_info,
+            parameters,
             project_info.points,
             range_changed,
             step_changed,
@@ -595,10 +604,49 @@ class SaveComponents(SaveBase):
 
     @classmethod
     def get_fields(cls) -> typing.List[typing.Union[AlgorithmProperty, str]]:
-        return []
+        return [AlgorithmProperty("frame", "Frame", 2), AlgorithmProperty("mask_data", "Mask data", True)]
+
+
+class SaveComponentsImagej(SaveBase):
+    @classmethod
+    def get_short_name(cls):
+        return "comp_imagej"
+
+    @classmethod
+    def save(
+        cls,
+        save_location: typing.Union[str, BytesIO, Path],
+        project_info: MaskProjectTuple,
+        parameters: dict,
+        range_changed=None,
+        step_changed=None,
+    ):
+        save_components(
+            project_info.image,
+            project_info.selected_components,
+            save_location,
+            project_info.roi_info,
+            parameters,
+            project_info.points,
+            range_changed,
+            step_changed,
+            writer_class=IMAGEJImageWriter,
+        )
+
+    @classmethod
+    def get_name(cls) -> str:
+        return "Components Imagej Tiff"
+
+    @classmethod
+    def get_fields(cls) -> typing.List[typing.Union[AlgorithmProperty, str]]:
+        return [AlgorithmProperty("frame", "Frame", 2), AlgorithmProperty("mask_data", "Mask data", True)]
 
 
 class SaveParametersJSON(SaveBase):
+    """
+    Save parameters of roi mask segmentation
+    """
+
     @classmethod
     def save(
         cls,
@@ -693,7 +741,7 @@ load_dict = Register(
     LoadStackImage, LoadROIImage, LoadStackImageWithMask, LoadPoints, class_methods=LoadBase.need_functions
 )
 save_parameters_dict = Register(SaveParametersJSON, class_methods=SaveBase.need_functions)
-save_components_dict = Register(SaveComponents, class_methods=SaveBase.need_functions)
+save_components_dict = Register(SaveComponents, SaveComponentsImagej, class_methods=SaveBase.need_functions)
 save_segmentation_dict = Register(
     SaveROI, SaveMaskAsTiff, SaveROIAsTIFF, SaveROIAsNumpy, class_methods=SaveBase.need_functions
 )
