@@ -55,6 +55,8 @@ class ROIExtractionResult:
     :ivar dict roi_annotation: Annotation for ROI. Currently displayed as tooltip
     :ivar Dict[str,np.ndarray] alternative_representation: Arrays with alternative representations of ROI.
     :ivar Optional[str] ~.file_path: information on which file roi extraction was performed.
+    :ivar ROIInfo ~.roi_info: ROIInfo for current roi.
+    :ivar Optional[np.ndarray] ~.points: array of points.
     """
 
     # TODO add alternative representation using dict mapping.
@@ -65,10 +67,8 @@ class ROIExtractionResult:
     roi_annotation: Dict = field(default_factory=dict)
     alternative_representation: Dict[str, np.ndarray] = field(default_factory=dict)
     file_path: Optional[str] = None
-
-    @property
-    def roi_info(self) -> ROIInfo:
-        return ROIInfo(roi=self.roi, annotations=self.roi_annotation, alternative=self.alternative_representation)
+    roi_info: Optional[ROIInfo] = None
+    points: Optional[np.ndarray] = None
 
     def __post_init__(self):
         if "ROI" in self.alternative_representation:
@@ -76,6 +76,12 @@ class ROIExtractionResult:
         for key, value in self.additional_layers.items():
             if value.name == "":
                 value.name = key
+        if self.roi_info is None:
+            object.__setattr__(
+                self,
+                "roi_info",
+                ROIInfo(roi=self.roi, annotations=self.roi_annotation, alternative=self.alternative_representation),
+            )
 
     def __str__(self):  # pragma: no cover
         return (
@@ -99,11 +105,11 @@ class ROIExtractionResult:
 SegmentationResult = ROIExtractionResult
 
 
-def report_empty_fun(_x, _y):  # pragma: no cover
+def report_empty_fun(_x, _y):  # pragma: no cover # skipcq: PTC-W0049
     pass
 
 
-class SegmentationAlgorithm(AlgorithmDescribeBase, ABC):
+class ROIExtractionAlgorithm(AlgorithmDescribeBase, ABC):
     """
     Base class for all segmentation algorithm.
 
@@ -211,10 +217,14 @@ class SegmentationAlgorithm(AlgorithmDescribeBase, ABC):
             return self.image.get_data_by_axis(c=channel_idx)
         if self.image.shape[self.image.time_pos] != 1:
             raise ValueError("This algorithm do not support time data")
-        if self.image.channels <= channel_idx:
+        if isinstance(channel_idx, int) and self.image.channels <= channel_idx:
             raise SegmentationException(
                 f"Image {self.image} has only {self.image.channels} when requested channel {channel_idx}"
             )
+        if isinstance(channel_idx, str) and channel_idx not in self.image.channel_names:
+            raise SegmentationException(
+                f"Image {self.image} has only {self.image.channel_names} when requested channel '{channel_idx}'"
+            )
         return self.image.get_data_by_axis(c=channel_idx, t=0)
 
     def set_image(self, image):
@@ -230,10 +240,8 @@ class SegmentationAlgorithm(AlgorithmDescribeBase, ABC):
             raise ValueError(f"Missed arguments {missed_arguments}; Additional arguments: {additional_arguments}")
         self.new_parameters = deepcopy(kwargs)
 
-    @abstractmethod
     def get_segmentation_profile(self) -> ROIExtractionProfile:
-        """Get parameters seated by :py:meth:`set_parameters` method."""
-        raise NotImplementedError()
+        return ROIExtractionProfile("", self.get_name(), deepcopy(self.new_parameters))
 
     @staticmethod
     def get_steps_num():
@@ -254,3 +262,6 @@ class SegmentationLimitException(Exception):
 
 class SegmentationException(Exception):
     pass
+
+
+SegmentationAlgorithm = ROIExtractionAlgorithm  # rename backward compatibility
