@@ -17,18 +17,18 @@ from ..mask_partition_utils import MaskDistanceSplit as MaskDistanceSplitBase
 from ..project_info import AdditionalLayerDescription
 from ..universal_const import Units
 from ..utils import bisect
-from .algorithm_base import ROIExtractionResult, SegmentationAlgorithm, SegmentationLimitException
+from .algorithm_base import ROIExtractionAlgorithm, ROIExtractionResult, SegmentationLimitException
 from .mu_mid_point import BaseMuMid, mu_mid_dict
 from .noise_filtering import noise_filtering_dict
 from .threshold import BaseThreshold, double_threshold_dict, threshold_dict
-from .watershed import BaseWatershed, calculate_distances_array, get_neigh, sprawl_dict
+from .watershed import BaseWatershed, calculate_distances_array, flow_dict, get_neigh
 
 
 def blank_operator(_x, _y):
     raise NotImplementedError()
 
 
-class RestartableAlgorithm(SegmentationAlgorithm, ABC):
+class RestartableAlgorithm(ROIExtractionAlgorithm, ABC):
     """
     Base class for restartable segmentation algorithm. The idea is to store two copies
     of algorithm parameters and base on difference check from which point restart the calculation.
@@ -255,7 +255,7 @@ class ThresholdBaseAlgorithm(RestartableAlgorithm, ABC):
             self.components_num = ind
             if ind == 0:
                 info_text = (
-                    "Please check the minimum size parameter. " f"The biggest element has size {self._sizes_array[1]}"
+                    f"Please check the minimum size parameter. The biggest element has size {self._sizes_array[1]}"
                 )
             else:
                 info_text = ""
@@ -400,8 +400,8 @@ class BaseThresholdFlowAlgorithm(TwoLevelThresholdBaseAlgorithm, ABC):
             AlgorithmProperty(
                 "sprawl_type",
                 "Flow type",
-                next(iter(sprawl_dict.keys())),
-                possible_values=sprawl_dict,
+                next(iter(flow_dict.keys())),
+                possible_values=flow_dict,
                 value_type=AlgorithmDescribeBase,
             ),
         )
@@ -460,7 +460,7 @@ class BaseThresholdFlowAlgorithm(TwoLevelThresholdBaseAlgorithm, ABC):
             if self.threshold_operator(self.threshold_info[1], self.threshold_info[0]):
                 self.final_sizes = np.bincount(finally_segment.flat)
                 return self.prepare_result(self.finally_segment)
-            path_sprawl: BaseWatershed = sprawl_dict[self.new_parameters["sprawl_type"]["name"]]
+            path_sprawl: BaseWatershed = flow_dict[self.new_parameters["sprawl_type"]["name"]]
             self.parameters["sprawl_type"] = self.new_parameters["sprawl_type"]
             new_segment = path_sprawl.sprawl(
                 self.sprawl_area,
