@@ -103,6 +103,8 @@ class Image:
         ranges=None,
         channel_names=None,
         axes_order: typing.Optional[str] = None,
+        shift: typing.Optional[Spacing] = None,
+        name: str = "",
     ):
         # TODO add time distance to image spacing
         if axes_order is None:
@@ -118,11 +120,16 @@ class Image:
         self._image_spacing = (1.0,) * (3 - len(image_spacing)) + image_spacing
         self._image_spacing = tuple(el if el > 0 else 10 ** -6 for el in self._image_spacing)
 
+        self._shift = tuple(shift) if shift is not None else (0,) * len(self._image_spacing)
+        self.name = name
+
         self.file_path = file_path
         self.default_coloring = default_coloring
         if self.default_coloring is not None:
             self.default_coloring = [np.array(x) for x in default_coloring]
         default_channel_names = [f"channel {i+1}" for i in range(self.channels)]
+        if isinstance(channel_names, str):
+            channel_names = [channel_names]
         if isinstance(channel_names, Iterable):
             self._channel_names = [
                 x if x is not None else y for x, y in zip_longest(channel_names, default_channel_names, fillvalue=None)
@@ -258,7 +265,7 @@ class Image:
         default_coloring=None,
         ranges=None,
         channel_names=None,
-    ):
+    ) -> "Image":
         """Create copy of image with substitution of not None elements"""
         data = self._image_array if data is None else data
         image_spacing = self._image_spacing if image_spacing is None else image_spacing
@@ -410,19 +417,21 @@ class Image:
         image_array = np.swapaxes(self._image_array, self.time_pos, self.stack_pos)
         return self.substitute(data=image_array)
 
-    def get_axis_positions(self) -> typing.Dict[str, int]:
+    @classmethod
+    def get_axis_positions(cls) -> typing.Dict[str, int]:
         """
         :return: dict with mapping axis to its position
         :rtype: dict
         """
-        return {l: i for i, l in enumerate(self.axis_order)}
+        return {l: i for i, l in enumerate(cls.axis_order)}
 
-    def get_array_axis_positions(self) -> typing.Dict[str, int]:
+    @classmethod
+    def get_array_axis_positions(cls) -> typing.Dict[str, int]:
         """
         :return: dict with mapping axis to its position for array fitted to image
         :rtype: dict
         """
-        return {l: i for i, l in enumerate(self.axis_order.replace("C", ""))}
+        return {l: i for i, l in enumerate(cls.axis_order.replace("C", ""))}
 
     def get_data_by_axis(self, **kwargs) -> np.ndarray:
         """
@@ -434,12 +443,22 @@ class Image:
         """
         slices: typing.List[typing.Union[int, slice]] = [slice(None) for _ in range(len(self.axis_order))]
         axis_pos = self.get_axis_positions()
+        if "c" in kwargs and isinstance(kwargs["c"], str):
+            kwargs["c"] = self.channel_names.index(kwargs["c"])
+        if "C" in kwargs and isinstance(kwargs["C"], str):
+            kwargs["C"] = self.channel_names.index(kwargs["C"])
         for name in kwargs:
             if name.upper() in axis_pos:
                 slices[axis_pos[name.upper()]] = kwargs[name]
         return self._image_array[tuple(slices)]
 
     def clip_array(self, array, **kwargs):
+        """
+        #TODO add docs
+        :param array:
+        :param kwargs:
+        :return:
+        """
         array = self.fit_array_to_image(array)
         slices: typing.List[typing.Union[int, slice]] = [slice(None) for _ in range(len(self.array_axis_order))]
         axis_pos = self.get_array_axis_positions()
@@ -452,7 +471,7 @@ class Image:
         """
         Alias for :py:func:`get_sub_data` with argument ``c=num``
 
-        :param int num: channel num to be extracted
+        :param int | str num: channel num to be extracted
         :return: given channel array
         :rtype: numpy.ndarray
         """
@@ -498,6 +517,12 @@ class Image:
             return (1, 1) + tuple(np.multiply(self.spacing, factor))
         return (1,) + tuple(np.multiply(self.spacing, factor))
 
+    @property
+    def shift(self):
+        if self.is_2d:
+            return self._shift[1:]
+        return self._shift
+
     @property
     def voxel_size(self) -> Spacing:
         """alias for spacing"""
@@ -514,78 +539,118 @@ class Image:
         self._image_spacing = tuple(value)
 
     @staticmethod
-    def _frame_array(array: typing.Optional[np.ndarray], index_to_add: typing.List[int]):
+    def _frame_array(array: typing.Optional[np.ndarray], index_to_add: typing.List[int], frame=FRAME_THICKNESS):
         if array is None:  # pragma: no cover
             return array
         result_shape = list(array.shape)
         image_pos = [slice(None) for _ in range(array.ndim)]
 
         for index in index_to_add:
-            result_shape[index] += FRAME_THICKNESS * 2
-            image_pos[index] = slice(FRAME_THICKNESS, result_shape[index] - FRAME_THICKNESS)
+            result_shape[index] += frame * 2
+            image_pos[index] = slice(frame, result_shape[index] - frame)
 
         data = np.zeros(shape=result_shape, dtype=array.dtype)
         data[tuple(image_pos)] = array
         return data
 
     @staticmethod
-    def calc_index_to_frame(array_axis, important_axis):
+    def calc_index_to_frame(array_axis: str, important_axis: str) -> typing.List[int]:
         """
         calculate in which axis frame should be added
 
         :param str array_axis: list of image axis
         :param str important_axis: list of framed axis
-        :return:
+        :return: list of indices to add frame.
         """
         return [array_axis.index(letter) for letter in important_axis]
 
+    def _frame_cut_area(self, cut_area: typing.Iterable[slice], frame: int):
+        cut_area = list(cut_area)
+        important_axis = "XY" if self.is_2d else "XYZ"
+        for ind in self.calc_index_to_frame(self.array_axis_order, important_axis):
+            sl = cut_area[ind]
+            cut_area[ind] = slice(
+                max(sl.start - frame, 0) if sl.start is not None else None,
+                sl.stop + frame if sl.stop is not None else None,
+                sl.step,
+            )
+        return cut_area
+
+    def _cut_image_slices(self, cut_area: typing.Iterable[slice], frame: int):
+        new_mask = None
+        cut_area = self._frame_cut_area(cut_area, frame)
+        cut_area2 = cut_area[:]
+        cut_area2.insert(self.channel_pos, slice(None))
+        new_image = self._image_array[tuple(cut_area2)]
+        if self._mask_array is not None:
+            new_mask = self._mask_array[tuple(cut_area)]
+        return new_image, new_mask
+
+    def _roi_to_slices(self, roi: np.ndarray) -> typing.List[slice]:
+        cut_area = self.fit_array_to_image(roi)
+        points = np.nonzero(cut_area)
+        lower_bound = np.min(points, axis=1)
+        upper_bound = np.max(points, axis=1)
+        return [slice(x, y + 1) for x, y in zip(lower_bound, upper_bound)]
+
+    def _cut_with_roi(self, cut_area: np.ndarray, replace_mask: bool, frame: int):
+        new_mask = None
+        cut_area = self.fit_array_to_image(cut_area)
+        new_cut = self._roi_to_slices(cut_area)
+        catted_cut_area = cut_area[tuple(new_cut)]
+        image_cut = new_cut[:]
+        image_cut.insert(self.channel_pos, slice(None))
+        new_image = np.copy(self._image_array[tuple(image_cut)])
+        if self.channel_pos == len(self.axis_order) - 1:
+            new_image[catted_cut_area == 0] = 0
+        else:
+            for i in range(self.channels):
+                np.take(new_image, i, self.channel_pos)[catted_cut_area == 0] = 0
+        if replace_mask:
+            new_mask = catted_cut_area
+        elif self._mask_array is not None:
+            new_mask = self._mask_array[tuple(new_cut)]
+            new_mask[catted_cut_area == 0] = 0
+        important_axis = "XY" if self.is_2d else "XYZ"
+        new_image = self._frame_array(new_image, self.calc_index_to_frame(self.axis_order, important_axis), frame)
+        new_mask = self._frame_array(new_mask, self.calc_index_to_frame(self.array_axis_order, important_axis), frame)
+        return new_image, new_mask
+
     def cut_image(
-        self, cut_area: typing.Union[np.ndarray, typing.List[slice], typing.Tuple[slice]], replace_mask=False
+        self,
+        cut_area: typing.Union[np.ndarray, typing.Iterable[slice]],
+        replace_mask=False,
+        frame: int = FRAME_THICKNESS,
+        zero_out_cut_area: bool = True,
     ) -> "Image":
         """
         Create new image base on mask or list of slices
-        :param replace_mask: if cut area is represented by mask array,
-        then in result image the mask is set base on cut_area
-        :param cut_area: area to cut. Defined with slices or mask
+        :param bool replace_mask: if cut area is represented by mask array,
+        then in result image the mask is set base on cut_area if cur_area is np.ndarray
+        :param typing.Union[np.ndarray, typing.Iterable[slice]] cut_area: area to cut. Defined with slices or mask
+        :param int frame: additional frame around cut_area
+        :param bool zero_out_cut_area:
         :return: Image
         """
-        new_mask = None
-        if isinstance(cut_area, (list, tuple)):
-            cut_area2 = cut_area[:]
-            cut_area2.insert(self.channel_pos, slice(None))
-            new_image = self._image_array[tuple(cut_area2)]
-            if self._mask_array is not None:
-                new_mask = self._mask_array[tuple(cut_area)]
-        else:
-            cut_area = self.fit_array_to_image(cut_area)
-            points = np.nonzero(cut_area)
-            lower_bound = np.min(points, axis=1)
-            upper_bound = np.max(points, axis=1)
-            new_cut = [slice(x, y + 1) for x, y in zip(lower_bound, upper_bound)]
-            catted_cut_area = cut_area[tuple(new_cut)]
-            image_cut = new_cut[:]
-            image_cut.insert(self.channel_pos, slice(None))
-            new_image = np.copy(self._image_array[tuple(image_cut)])
-            if self.channel_pos == len(self.axis_order) - 1:
-                new_image[catted_cut_area == 0] = 0
+        if isinstance(cut_area, np.ndarray):
+            if zero_out_cut_area:
+                new_image, new_mask = self._cut_with_roi(cut_area, replace_mask, frame)
             else:
-                for i in range(self.channels):
-                    np.take(new_image, i, self.channel_pos)[catted_cut_area == 0] = 0
-            if replace_mask:
-                new_mask = catted_cut_area
-            elif self._mask_array is not None:
-                new_mask = self._mask_array[tuple(new_cut)]
-                new_mask[catted_cut_area == 0] = 0
-        important_axis = "XY" if self.is_2d else "XYZ"
+                new_cut = self._roi_to_slices(cut_area)
+                new_image, new_mask = self._cut_image_slices(new_cut, frame)
+                if replace_mask:
+                    new_mask = cut_area[tuple(self._frame_cut_area(new_cut, frame))]
+        else:
+            new_image, new_mask = self._cut_image_slices(cut_area, frame)
 
         return self.__class__(
-            self._frame_array(new_image, self.calc_index_to_frame(self.axis_order, important_axis)),
-            self._image_spacing,
-            None,
-            self._frame_array(new_mask, self.calc_index_to_frame(self.array_axis_order, important_axis)),
-            self.default_coloring,
-            self.ranges,
-            self.channel_names,
+            data=new_image,
+            image_spacing=self._image_spacing,
+            file_path=None,
+            mask=new_mask,
+            default_coloring=self.default_coloring,
+            ranges=self.ranges,
+            channel_names=self.channel_names,
         )
 
     def get_imagej_colors(self):
@@ -630,6 +695,10 @@ class Image:
         """image spacing in micrometers"""
         return tuple(float(x * 10 ** 6) for x in self.spacing)
 
+    def get_um_shift(self) -> Spacing:
+        """image spacing in micrometers"""
+        return tuple(float(x * 10 ** 6) for x in self.shift)
+
     def get_ranges(self) -> typing.List[typing.Tuple[float, float]]:
         """image brightness ranges for each channel"""
         return self.ranges[:]
