@@ -1,8 +1,11 @@
 import dataclasses
 import itertools
+import multiprocessing as mp
 import os
+import signal
 from copy import deepcopy
 from pathlib import Path
+from queue import Empty
 
 import numpy as np
 import pytest
@@ -21,7 +24,7 @@ from PartSegImage import Image
 @pytest.fixture(scope="module")
 def data_test_dir():
     """Return path to directory with test data"""
-    return os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(__file__))), "test_data")
+    return os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))), "test_data")
 
 
 @pytest.fixture(scope="module")
@@ -30,6 +33,34 @@ def bundle_test_dir():
     return Path(os.path.join(os.path.dirname(__file__), "test_data"))
 
 
+def wait_sigint(q: mp.Queue, pid):
+    try:
+        q.get(timeout=20 * 60)
+    except Empty:
+        print("Timeout")
+        os.kill(pid, signal.SIGINT)
+        import time
+
+        time.sleep(30)
+        try:
+            os.kill(pid, 0)
+        except OSError:
+            pass
+        else:
+            os.kill(pid, signal.SIGKILL)
+
+
+@pytest.fixture(scope="session", autouse=True)
+def sigint_after_time():
+    manager = mp.Manager()
+    q = manager.Queue()
+    p = mp.Process(target=wait_sigint, args=(q, os.getpid()))
+    p.start()
+    yield
+    q.put(1)
+    p.join()
+
+
 @pytest.fixture
 def image():
     data = np.zeros([20, 20, 20, 2], dtype=np.uint8)
@@ -158,3 +189,10 @@ def measurement_profiles():
         ),
     ]
     return MeasurementProfile("statistic1", statistics), MeasurementProfile("statistic2", statistics + statistics2)
+
+
+def pytest_collection_modifyitems(session, config, items):
+    image_tests = [x for x in items if "PartSegImage" in str(x.fspath)]
+    core_tests = [x for x in items if "PartSegCore" in str(x.fspath)]
+    other_test = [x for x in items if "PartSegCore" not in str(x.fspath) and "PartSegImage" not in str(x.fspath)]
+    items[:] = image_tests + core_tests + other_test
