@@ -18,29 +18,29 @@ class TestProperties:
     def test_pipeline_profile_show_info(
         self, qtbot, part_settings, border_rim_profile, lower_threshold_profile, sample_pipeline
     ):
-        part_settings.segmentation_profiles[border_rim_profile.name] = border_rim_profile
-        part_settings.segmentation_pipelines[sample_pipeline.name] = sample_pipeline
+        part_settings.roi_profiles[border_rim_profile.name] = border_rim_profile
+        part_settings.roi_pipelines[sample_pipeline.name] = sample_pipeline
         widget = Properties(part_settings)
         widget.show()
         qtbot.addWidget(widget)
         widget.update_profile_list()
         assert widget.profile_list.count() == 1
-        part_settings.segmentation_profiles[lower_threshold_profile.name] = lower_threshold_profile
+        part_settings.roi_profiles[lower_threshold_profile.name] = lower_threshold_profile
         widget.update_profile_list()
         assert widget.profile_list.count() == 2
         assert widget.pipeline_list.count() == 1
         assert widget.info_label.toPlainText() == ""
         with qtbot.waitSignal(widget.profile_list.currentItemChanged, timeout=10 ** 4):
             widget.profile_list.setCurrentRow(1)
-        profile = part_settings.segmentation_profiles[widget.profile_list.item(1).text()]
+        profile = part_settings.roi_profiles[widget.profile_list.item(1).text()]
         assert widget.info_label.toPlainText() == profile.pretty_print(analysis_algorithm_dict)
         widget.pipeline_list.setCurrentRow(0)
         assert widget.info_label.toPlainText() == sample_pipeline.pretty_print(analysis_algorithm_dict)
         widget.hide()
 
     def test_delete_profile(self, qtbot, part_settings, border_rim_profile, lower_threshold_profile):
-        part_settings.segmentation_profiles[border_rim_profile.name] = border_rim_profile
-        part_settings.segmentation_profiles[lower_threshold_profile.name] = lower_threshold_profile
+        part_settings.roi_profiles[border_rim_profile.name] = border_rim_profile
+        part_settings.roi_profiles[lower_threshold_profile.name] = lower_threshold_profile
         widget = Properties(part_settings)
         widget.show()
         qtbot.addWidget(widget)
@@ -51,16 +51,16 @@ class TestProperties:
         assert widget.delete_btn.isEnabled()
         with qtbot.waitSignal(widget.delete_btn.clicked):
             widget.delete_btn.click()
-        assert len(part_settings.segmentation_profiles) == 1
-        assert lower_threshold_profile.name in part_settings.segmentation_profiles
+        assert len(part_settings.roi_profiles) == 1
+        assert lower_threshold_profile.name in part_settings.roi_profiles
         widget.hide()
 
     def test_rename_profile(
         self, qtbot, part_settings, border_rim_profile, lower_threshold_profile, sample_pipeline, monkeypatch
     ):
-        part_settings.segmentation_profiles[border_rim_profile.name] = border_rim_profile
-        part_settings.segmentation_pipelines[sample_pipeline.name] = sample_pipeline
-        part_settings.segmentation_profiles[lower_threshold_profile.name] = lower_threshold_profile
+        part_settings.roi_profiles[border_rim_profile.name] = border_rim_profile
+        part_settings.roi_pipelines[sample_pipeline.name] = sample_pipeline
+        part_settings.roi_profiles[lower_threshold_profile.name] = lower_threshold_profile
         widget = Properties(part_settings)
         widget.show()
         qtbot.addWidget(widget)
@@ -70,12 +70,12 @@ class TestProperties:
         widget.profile_list.setCurrentRow(0)
         widget.rename_profile()
         assert widget.profile_list.item(1).text() == "rim"
-        assert set(part_settings.segmentation_profiles.keys()) == {"rim", lower_threshold_profile.name}
+        assert set(part_settings.roi_profiles.keys()) == {"rim", lower_threshold_profile.name}
         monkeypatch.setattr(advanced_module_input, "getText", check_text(sample_pipeline.name, "rim"))
         widget.pipeline_list.setCurrentRow(0)
         widget.rename_profile()
         assert widget.pipeline_list.item(0).text() == "rim"
-        assert set(part_settings.segmentation_pipelines.keys()) == {"rim"}
+        assert set(part_settings.roi_pipelines.keys()) == {"rim"}
         monkeypatch.setattr(advanced_module_input, "getText", check_text("rim", lower_threshold_profile.name))
 
         called_mock = [0]
@@ -90,7 +90,7 @@ class TestProperties:
         widget.rename_profile()
         assert called_mock[0] == 1
         assert widget.profile_list.item(1).text() == "rim"
-        assert set(part_settings.segmentation_profiles.keys()) == {"rim", lower_threshold_profile.name}
+        assert set(part_settings.roi_profiles.keys()) == {"rim", lower_threshold_profile.name}
         widget.hide()
 
 
