@@ -36,9 +36,8 @@ def test_color_combo_box(qtbot):
     box = ColorComboBox(0, starting_colors, dkt)
     box.show()
     qtbot.add_widget(box)
-    with qtbot.waitSignal(box.channel_visible_changed):
-        with qtbot.assertNotEmitted(box.clicked):
-            qtbot.mouseClick(box.check_box, Qt.LeftButton)
+    with qtbot.waitSignal(box.channel_visible_changed), qtbot.assertNotEmitted(box.clicked):
+        qtbot.mouseClick(box.check_box, Qt.LeftButton)
     with qtbot.waitSignal(box.clicked, timeout=1000):
         qtbot.mouseClick(box, Qt.LeftButton, pos=QPoint(5, 5))
     with qtbot.waitSignal(box.clicked):
@@ -138,12 +137,12 @@ class TestColorComboBoxGroup:
         with qtbot.waitSignal(box.coloring_update), qtbot.waitSignal(
             box.change_channel, check_params_cb=check_parameters
         ):
-            ch_property.use_filter.set_value(NoiseFilterType.Gauss)
+            ch_property.use_filter.setCurrentEnum(NoiseFilterType.Gauss)
 
         with qtbot.waitSignal(box.coloring_update), qtbot.waitSignal(
             box.change_channel, check_params_cb=check_parameters
         ):
-            ch_property.use_filter.set_value(NoiseFilterType.Median)
+            ch_property.use_filter.setCurrentEnum(NoiseFilterType.Median)
 
         ch_property.filter_radius.setValue(0.5)
         with qtbot.waitSignal(box.coloring_update), qtbot.waitSignal(
@@ -154,7 +153,7 @@ class TestColorComboBoxGroup:
         with qtbot.waitSignal(box.coloring_update), qtbot.waitSignal(
             box.change_channel, check_params_cb=check_parameters
         ):
-            ch_property.use_filter.set_value(NoiseFilterType.No)
+            ch_property.use_filter.setCurrentEnum(NoiseFilterType.No)
 
         with qtbot.assert_not_emitted(box.coloring_update), qtbot.assert_not_emitted(box.change_channel):
             ch_property.filter_radius.setValue(0.5)
@@ -226,7 +225,7 @@ class TestColorComboBoxGroup:
         with qtbot.waitSignal(image_view.channel_control.coloring_update), qtbot.waitSignal(
             image_view.channel_control.change_channel, check_params_cb=check_parameters
         ):
-            ch_property.use_filter.set_value(NoiseFilterType.Gauss)
+            ch_property.use_filter.setCurrentEnum(NoiseFilterType.Gauss)
         image4 = image_view.viewer_widget.screenshot()
         assert np.any(image4 != 255)
         assert np.any(image1 != image4)
