@@ -35,6 +35,29 @@ def test_sentry_serialize_clip(monkeypatch):
         assert len(cliped["message"]) == 5000
 
 
+def test_sentry_variables_clip(monkeypatch):
+    letters = "abcdefghijklmnoprst"
+    for letter in letters:
+        locals()[letter] = 1
+    try:
+        raise ValueError("eeee")
+    except ValueError as ee:
+        event, hint = sentry_sdk.utils.event_from_exception(ee)
+        vars_dict = event["exception"]["values"][0]["stacktrace"]["frames"][0]["vars"]
+        for letter in letters:
+            assert letter in vars_dict
+        cliped = serialize(event)
+        assert len(cliped["exception"]["values"][0]["stacktrace"]["frames"][0]["vars"]) == 10
+        monkeypatch.setattr(sentry_sdk.serializer, "MAX_DATABAG_BREADTH", 100)
+        cliped = serialize(event)
+        assert len(cliped["exception"]["values"][0]["stacktrace"]["frames"][0]["vars"]) == len(vars_dict)
+        assert len(cliped["exception"]["values"][0]["stacktrace"]["frames"][0]["vars"]) > 10
+
+        client = Client("https://aaa@test.pl/77")
+        Hub.current.bind_client(client)
+        sentry_sdk.capture_event(event, hint=hint)
+
+
 def test_sentry_report(monkeypatch):
     message = "a" * 5000
     happen = [False]
