@@ -10,16 +10,20 @@ import numpy as np
 import pytest
 from sympy import symbols
 
+from PartSegCore.algorithm_describe_base import ROIExtractionProfile
 from PartSegCore.analysis import load_metadata
 from PartSegCore.analysis.measurement_base import AreaType, Leaf, MeasurementEntry, Node, PerComponent
 from PartSegCore.analysis.measurement_calculation import (
     HARALIC_FEATURES,
     MEASUREMENT_DICT,
+    ColocalizationMeasurement,
     ComponentsInfo,
     ComponentsNumber,
+    CorrelationEnum,
     Diameter,
-    DistanceMaskSegmentation,
+    DistanceMaskROI,
     DistancePoint,
+    DistanceROIROI,
     FirstPrincipalAxisLength,
     Haralick,
     MaximumPixelBrightness,
@@ -32,6 +36,7 @@ from PartSegCore.analysis.measurement_calculation import (
     PixelBrightnessSum,
     RimPixelBrightnessSum,
     RimVolume,
+    ROINeighbourhoodROI,
     SecondPrincipalAxisLength,
     Sphericity,
     SplitOnPartPixelBrightnessSum,
@@ -43,6 +48,8 @@ from PartSegCore.analysis.measurement_calculation import (
     Voxels,
 )
 from PartSegCore.autofit import density_mass_center
+from PartSegCore.roi_info import ROIInfo
+from PartSegCore.segmentation.restartable_segmentation_algorithms import LowerThresholdAlgorithm
 from PartSegCore.universal_const import UNIT_SCALE, Units
 from PartSegImage import Image
 
@@ -175,9 +182,11 @@ class TestPixelBrightnessSum:
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
         mask3 = mask1 * ~mask2
-        assert PixelBrightnessSum.calculate_property(mask1, image.get_channel(0)) == 60 * 60 * 50 + 40 * 40 * 20
-        assert PixelBrightnessSum.calculate_property(mask2, image.get_channel(0)) == 40 * 40 * 70
-        assert PixelBrightnessSum.calculate_property(mask3, image.get_channel(0)) == (60 * 60 - 40 * 40) * 50
+        assert PixelBrightnessSum.calculate_property(mask1, image.get_channel(0)) == 60 ** 2 * 50 + 40 * 40 * 20
+
+        assert PixelBrightnessSum.calculate_property(mask2, image.get_channel(0)) == 40 ** 2 * 70
+
+        assert PixelBrightnessSum.calculate_property(mask3, image.get_channel(0)) == (60 ** 2 - 40 ** 2) * 50
 
     def test_empty(self):
         image = get_cube_image()
@@ -258,7 +267,7 @@ class TestVoxels:
         mask3 = mask1 * ~mask2
         assert Voxels.calculate_property(mask1) == 60 * 60
         assert Voxels.calculate_property(mask2) == 40 * 40
-        assert Voxels.calculate_property(mask3) == 60 * 60 - 40 * 40
+        assert Voxels.calculate_property(mask3) == 60 ** 2 - 40 ** 2
 
     def test_empty(self):
         image = get_cube_image()
@@ -786,10 +795,10 @@ def two_comp_img():
 
 class TestDistanceMaskSegmentation:
     def test_parameters(self):
-        assert DistanceMaskSegmentation.get_units(3) == symbols("{}")
-        assert DistanceMaskSegmentation.get_units(2) == symbols("{}")
-        assert DistanceMaskSegmentation.need_channel() is True
-        leaf = DistanceMaskSegmentation.get_starting_leaf()
+        assert DistanceMaskROI.get_units(3) == symbols("{}")
+        assert DistanceMaskROI.get_units(2) == symbols("{}")
+        assert DistanceMaskROI.need_channel() is True
+        leaf = DistanceMaskROI.get_starting_leaf()
         assert isinstance(leaf, Leaf)
         assert leaf.area is AreaType.Mask
         assert leaf.per_component is None
@@ -802,7 +811,7 @@ class TestDistanceMaskSegmentation:
         mask1 = cube_image.get_channel(0)[0] > 40
         mask2 = cube_image.get_channel(0)[0] > 60
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 channel=cube_image.get_channel(0),
                 area_array=mask2,
                 mask=mask1,
@@ -827,7 +836,7 @@ class TestDistanceMaskSegmentation:
         mask2 = cube_image.get_channel(0)[0] > 60
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 channel=cube_image.get_channel(0),
                 area_array=mask2,
                 mask=mask1,
@@ -859,7 +868,7 @@ class TestDistanceMaskSegmentation:
         else:
             area_mid = np.average(np.nonzero(area_array), axis=1, weights=channel[0][area_array])
         assert isclose(
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 channel=channel,
                 area_array=area_array,
                 mask=mask,
@@ -876,7 +885,7 @@ class TestDistanceMaskSegmentation:
         mask[2:-2, 2:-2, 2:-2] = 1
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 two_comp_img.get_channel(0),
                 two_comp_img.get_channel(0)[0],
                 mask,
@@ -889,7 +898,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 two_comp_img.get_channel(0),
                 two_comp_img.get_channel(0)[0],
                 mask,
@@ -902,7 +911,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 two_comp_img.get_channel(0),
                 two_comp_img.get_channel(0)[0],
                 mask,
@@ -915,7 +924,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 two_comp_img.get_channel(0),
                 two_comp_img.get_channel(0)[0] == 50,
                 mask,
@@ -928,7 +937,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 two_comp_img.get_channel(0),
                 two_comp_img.get_channel(0)[0] == 60,
                 mask,
@@ -945,7 +954,7 @@ class TestDistanceMaskSegmentation:
         mask1 = image.get_channel(0)[0] > 40
         mask2 = image.get_channel(0)[0] > 60
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 image.get_channel(0),
                 mask2,
                 mask1,
@@ -961,7 +970,7 @@ class TestDistanceMaskSegmentation:
         mask3[mask2 == 0] = 0
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 image.get_channel(0),
                 mask2,
                 mask1,
@@ -974,7 +983,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 mask3,
                 mask3 == 1,
                 mask1,
@@ -987,7 +996,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 mask3,
                 mask3 == 2,
                 mask1,
@@ -1000,7 +1009,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 mask3,
                 mask3 == 1,
                 mask1,
@@ -1013,7 +1022,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert (
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 mask3,
                 mask3 == 2,
                 mask1,
@@ -1026,7 +1035,7 @@ class TestDistanceMaskSegmentation:
         )
 
         assert isclose(
-            DistanceMaskSegmentation.calculate_property(
+            DistanceMaskROI.calculate_property(
                 mask3, mask2, mask1, image.voxel_size, 1, DistancePoint.Geometrical_center, DistancePoint.Mass_center
             ),
             1000 * 2 / 3 - 500,
@@ -1228,7 +1237,7 @@ class TestSplitOnPartVolume:
                 voxel_size=image.voxel_size,
                 result_scalar=1,
             )
-            == (60 * 60 - 40 * 40) * result_scale
+            == (60 ** 2 - 40 ** 2) * result_scale
         )
 
         assert (
@@ -1241,7 +1250,7 @@ class TestSplitOnPartVolume:
                 voxel_size=image.voxel_size,
                 result_scalar=1,
             )
-            == (60 * 60 - 30 * 30) * result_scale
+            == (60 * 60 - 30 ** 2) * result_scale
         )
 
         assert (
@@ -1301,7 +1310,7 @@ class TestSplitOnPartVolume:
                 voxel_size=image.voxel_size,
                 result_scalar=1,
             )
-            == (60 * 60 - 44 * 44) * result_scale
+            == (60 ** 2 - 44 * 44) * result_scale
         )
 
         assert (
@@ -1426,10 +1435,10 @@ class TestSplitOnPartPixelBrightnessSum:
         "nr, sum_val, diff_array, equal_volume",
         [
             (3, (60 * 60 - 40 * 40) * 50, False, False),
-            (2, (60 * 60 - 40 * 40) * 50 + (40 * 40 - 30 * 30) * 70, False, False),
+            (2, (60 ** 2 - 40 * 40) * 50 + (40 * 40 - 30 * 30) * 70, False, False),
             (3, 0, True, False),
-            (2, (40 * 40 - 30 * 30) * 70, True, False),
-            (3, (60 * 60 - 50 * 50) * 50, False, True),
+            (2, (40 ** 2 - 30 ** 2) * 70, True, False),
+            (3, (60 * 60 - 50 ** 2) * 50, False, True),
             (2, (60 * 60 - 44 * 44) * 50, False, True),
             (3, 0, True, True),
             (2, 0, True, True),
@@ -1817,6 +1826,12 @@ class TestStatisticProfile:
         assert len(result["Mask Volume per component/ROI Volume per component"][0]) == 2
         assert len(result["Mask Volume per component/Mask without ROI Volume per component"][0]) == 1
         assert len(result["Density per component"][0]) == 2
+        assert np.all(
+            np.isclose(
+                result["Mask Volume per component/ROI Volume per component"][0],
+                1 / np.array(result["ROI Volume per component/Mask Volume per component"][0]),
+            )
+        )
 
 
 # noinspection DuplicatedCode
@@ -2001,24 +2016,237 @@ class TestHaralick:
         Haralick.calculate_property(mask, data, distance=distance, feature=feature)
 
 
+@pytest.fixture
+def roi_to_roi_extract():
+    parameters = LowerThresholdAlgorithm.get_default_values()
+    parameters["threshold"]["values"]["threshold"] = 1
+    parameters["minimum_size"] = 1
+    parameters["channel"] = 1
+    return ROIExtractionProfile("default", LowerThresholdAlgorithm.get_name(), parameters)
+
+
+@pytest.mark.parametrize("roi_dist", DistancePoint.__members__.values())
+@pytest.mark.parametrize("new_roi_dist", DistancePoint.__members__.values())
+class TestDistanceROIROI:
+    def test_base(self, roi_dist, new_roi_dist, roi_to_roi_extract):
+        data = np.zeros((3, 10, 10, 20), dtype=np.uint8)
+        data[0, 2:-2, 2:-2, 2:-12] = 5
+        data[1, 2:-2, 2:-2, 12:-2] = 5
+        data[2, 2:-2, 2:-2, 2:-2] = 5
+        image = Image(data, image_spacing=(1, 1, 1), axes_order="CZYX")
+        roi = (data[0] > 1).astype(np.uint8)
+        res = DistanceROIROI.calculate_property(
+            channel=data[2],
+            image=image,
+            area_array=roi,
+            profile=roi_to_roi_extract,
+            mask=None,
+            voxel_size=image.voxel_size,
+            result_scalar=1,
+            distance_from_new_roi=new_roi_dist,
+            distance_to_roi=roi_dist,
+        )
+        assert res > 0
+        data[1, 3:-3, 3:-3, 3:-13] = 5
+        image = Image(data, image_spacing=(1, 1, 1), axes_order="CZYX")
+        roi = (data[0] > 1).astype(np.uint8)
+        res = DistanceROIROI.calculate_property(
+            channel=data[2],
+            image=image,
+            area_array=roi,
+            profile=roi_to_roi_extract,
+            mask=None,
+            voxel_size=image.voxel_size,
+            result_scalar=1,
+            distance_from_new_roi=new_roi_dist,
+            distance_to_roi=roi_dist,
+        )
+        assert res == 0
+
+    def test_base_2d(self, roi_dist, new_roi_dist, roi_to_roi_extract):
+        data = np.zeros((3, 10, 20), dtype=np.uint8)
+        data[0, 2:-2, 2:-12] = 5
+        data[1, 2:-2, 12:-2] = 5
+        data[2, 2:-2, 2:-2] = 5
+        image = Image(data, image_spacing=(1, 1, 1), axes_order="CYX")
+        roi = (data[0:1] > 1).astype(np.uint8)
+        res = DistanceROIROI.calculate_property(
+            channel=data[2:3],
+            image=image,
+            area_array=roi,
+            profile=roi_to_roi_extract,
+            mask=None,
+            voxel_size=image.voxel_size,
+            result_scalar=1,
+            distance_from_new_roi=new_roi_dist,
+            distance_to_roi=roi_dist,
+        )
+        assert res > 0
+
+
+class TestROINeighbourhoodROI:
+    def test_base(self, roi_to_roi_extract):
+        data = np.zeros((3, 10, 10, 20), dtype=np.uint8)
+        data[0, 2:-2, 2:-2, 2:-12] = 5
+        data[1, 2:-2, 2:-2, 12:-2] = 5
+        data[2, 2:-2, 2:-2, 2:-2] = 5
+        image = Image(data, image_spacing=(100 * (10 ** -9),) * 3, axes_order="CZYX")
+        roi = (data[0] > 1).astype(np.uint8)
+        kwargs = {
+            "image": image,
+            "area_array": roi,
+            "profile": roi_to_roi_extract,
+            "mask": None,
+            "voxel_size": image.voxel_size,
+            "distance": 100,
+            "units": Units.nm,
+        }
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 0
+        kwargs["distance"] = 1000
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 1
+        data[1, 3:-3, 3:-3, 3:10] = 5
+        image = Image(data, image_spacing=(100 * (10 ** -9),) * 3, axes_order="CZYX")
+        kwargs["image"] = image
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 2
+        kwargs["distance"] = 100
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 1
+
+    def test_base2d(self, roi_to_roi_extract):
+        data = np.zeros((3, 10, 20), dtype=np.uint8)
+        data[0, 2:-2, 2:-12] = 5
+        data[1, 2:-2, 12:-2] = 5
+        data[2, 2:-2, 2:-2] = 5
+        image = Image(data, image_spacing=(100 * (10 ** -9),) * 2, axes_order="CYX")
+        roi = (data[0:1] > 1).astype(np.uint8)
+        kwargs = {
+            "image": image,
+            "area_array": roi,
+            "profile": roi_to_roi_extract,
+            "mask": None,
+            "voxel_size": image.voxel_size,
+            "distance": 100,
+            "units": Units.nm,
+        }
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 0
+        kwargs["distance"] = 1000
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 1
+        data[1, 3:-3, 3:10] = 5
+        image = Image(data, image_spacing=(100 * (10 ** -9),) * 2, axes_order="CYX")
+        kwargs["image"] = image
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 2
+        kwargs["distance"] = 100
+        assert ROINeighbourhoodROI.calculate_property(**kwargs) == 1
+
+
 @pytest.mark.parametrize("method", MEASUREMENT_DICT.values())
 @pytest.mark.parametrize("dtype", [float, int, np.uint8, np.uint16, np.uint32, np.float16, np.float32])
-def ttest_all_methods(method, dtype):
+def test_all_methods(method, dtype):
     data = np.zeros((10, 20, 20), dtype=dtype)
     data[1:-1, 3:-3, 3:-3] = 2
     data[1:-1, 4:-4, 4:-4] = 3
-    roi = (data > 0).astype(np.uint8)
-    mask = (data > 2).astype(np.uint8)
+    roi = (data > 2).astype(np.uint8)
+    mask = (data > 0).astype(np.uint8)
+    roi_info = ROIInfo(roi)
+    image = Image(data, image_spacing=(1, 1, 1), axes_order="ZYX")
 
     res = method.calculate_property(
+        image=image,
         area_array=roi,
         mask=mask,
         channel=data,
         channel_num=0,
+        channel_0=data,
+        channel_1=data,
         voxel_size=(1, 1, 1),
         result_scalar=1,
         roi_alternative={},
         roi_annotation={},
+        bounds_info=roi_info.bound_info,
+        _component_num=1,
         **method.get_default_values(),
     )
-    float(res)
+    if method.get_units(3) != "str":
+        float(res)
+
+
+@pytest.mark.parametrize(
+    "method", (x for x in MEASUREMENT_DICT.values() if x.get_starting_leaf().per_component is None)
+)
+@pytest.mark.parametrize("area", [AreaType.ROI, AreaType.Mask])
+def test_per_component(method, area):
+    data = np.zeros((10, 20, 20, 2), dtype=np.uint8)
+    data[1:-1, 3:-3, 3:-3] = 2
+    data[1:-1, 4:-4, 4:-4] = 3
+    data[1:-1, 6, 6] = 5
+    roi = (data[..., 0] > 2).astype(np.uint8)
+    mask = (data[..., 0] > 0).astype(np.uint8)
+    image = Image(data, image_spacing=(10 ** -8,) * 3, axes_order="ZYXC")
+    image.set_mask(mask, axes="ZYX")
+
+    statistics = [
+        MeasurementEntry(
+            "Measurement",
+            method.get_starting_leaf().replace_(
+                per_component=PerComponent.No, area=area, dict=method.get_default_values()
+            ),
+        ),
+        MeasurementEntry(
+            "Measurement per component",
+            method.get_starting_leaf().replace_(
+                per_component=PerComponent.Yes, area=area, dict=method.get_default_values()
+            ),
+        ),
+    ]
+    profile = MeasurementProfile("statistic", statistics)
+    result = profile.calculate(
+        image,
+        0,
+        roi,
+        result_units=Units.nm,
+    )
+    assert len(result["Measurement per component"][0]) == 1
+    assert isinstance(result["Measurement"][0], (float, int))
+    assert result["Measurement per component"][0][0] == result["Measurement"][0]
+
+
+@pytest.mark.parametrize("method", CorrelationEnum.__members__.values())
+@pytest.mark.parametrize("randomize", [True, False])
+def test_colocalization(method, randomize):
+    area_array = np.ones((10, 10))
+    data = np.random.rand(10, 10)
+    factor = 0.5 if method == CorrelationEnum.intensity else 1
+    value = ColocalizationMeasurement.calculate_property(
+        area_array=area_array,
+        channel_0=data,
+        channel_1=data,
+        colocalization=method,
+        randomize=randomize,
+    )
+    assert value == factor or randomize
+    value = ColocalizationMeasurement.calculate_property(
+        area_array=area_array,
+        channel_0=data,
+        channel_1=data * 100,
+        colocalization=method,
+        randomize=randomize,
+    )
+    assert isclose(value, factor) or randomize
+
+    value = ColocalizationMeasurement.calculate_property(
+        area_array=area_array,
+        channel_0=data,
+        channel_1=data + 100,
+        colocalization=method,
+        randomize=randomize,
+    )
+
+    assert isclose(value, factor) or (method == CorrelationEnum.manders and value < 1) or randomize
+
+    value = ColocalizationMeasurement.calculate_property(
+        area_array=area_array,
+        channel_0=data,
+        channel_1=-data,
+        colocalization=method,
+        randomize=randomize,
+    )
+    assert value == -factor or randomize
