@@ -1,3 +1,5 @@
+# pylint: disable=R0201
+
 import itertools
 
 import numpy as np
@@ -101,9 +103,9 @@ class TestMu:
         assert np.all(res == (np.ones(image.shape, dtype=np.float64) - (image == 10) * 0.5) * mask)
 
     def test_two_object_mu(self):
-        pass
-        # TODO
-        # image = np.zeros((10, 10, 10), dtype=np.uint8)
+        """
+        TODO add this test
+        """
 
     def test_reshape(self):
         image = np.zeros((40, 150, 120), dtype=np.uint16)
@@ -557,9 +559,9 @@ class TestMuMid:
         res = calculate_mu_mid(data, 5, 10, 20)
         assert np.all(ones == res)
         res = calculate_mu_mid(data, 5, 20, 30)
-        assert np.all(res == (data == 0).astype(np.float))
+        assert np.all(res == (data == 0).astype(float))
         data[2:-2, 2:-2, 2:-2] = 30
         res = calculate_mu_mid(data, 5, 25, 30)
-        assert np.all(res == (data != 20).astype(np.float) + (data == 20) * 0.25)
+        assert np.all(res == (data != 20).astype(float) + (data == 20) * 0.25)
         res = calculate_mu_mid(data, 5, 25, 35)
-        assert np.all(res == (data == 0).astype(np.float) + (data == 20) * 0.25 + (data == 30) * 0.5)
+        assert np.all(res == (data == 0).astype(float) + (data == 20) * 0.25 + (data == 30) * 0.5)
