@@ -16,10 +16,10 @@ from PartSegCore.convex_fill import _convex_fill, convex_fill
 from PartSegCore.image_operations import RadiusType
 from PartSegCore.mask_create import MaskProperty, calculate_mask
 from PartSegCore.roi_info import BoundInfo, ROIInfo
-from PartSegCore.segmentation import SegmentationAlgorithm, algorithm_base
+from PartSegCore.segmentation import ROIExtractionAlgorithm, algorithm_base
 from PartSegCore.segmentation import restartable_segmentation_algorithms as sa
 from PartSegCore.segmentation.noise_filtering import noise_filtering_dict
-from PartSegCore.segmentation.watershed import sprawl_dict
+from PartSegCore.segmentation.watershed import flow_dict
 from PartSegImage import Image
 
 
@@ -84,7 +84,7 @@ def empty(_s: str, _i: int):
 def test_base_parameters(algorithm_name):
     algorithm_class = analysis_algorithm_dict[algorithm_name]
     assert algorithm_class.get_name() == algorithm_name
-    algorithm_class: Type[SegmentationAlgorithm]
+    algorithm_class: Type[ROIExtractionAlgorithm]
     obj = algorithm_class()
     values = algorithm_class.get_default_values()
     obj.set_parameters(**values)
@@ -118,14 +118,14 @@ class BaseThreshold:
     def get_side_object():
         raise NotImplementedError
 
-    def get_algorithm_class(self) -> Type[SegmentationAlgorithm]:
+    def get_algorithm_class(self) -> Type[ROIExtractionAlgorithm]:
         raise NotImplementedError()
 
 
 class BaseOneThreshold(BaseThreshold, ABC):  # pylint: disable=W0223
     def test_simple(self):
         image = self.get_base_object()
-        alg: SegmentationAlgorithm = self.get_algorithm_class()()
+        alg: ROIExtractionAlgorithm = self.get_algorithm_class()()
         parameters = self.get_parameters()
         alg.set_image(image)
         alg.set_parameters(**parameters)
@@ -139,7 +139,7 @@ class BaseOneThreshold(BaseThreshold, ABC):  # pylint: disable=W0223
 
     def test_side_connection(self):
         image = self.get_side_object()
-        alg: SegmentationAlgorithm = self.get_algorithm_class()()
+        alg: ROIExtractionAlgorithm = self.get_algorithm_class()()
         parameters = self.get_parameters()
         parameters["side_connection"] = True
         alg.set_image(image)
@@ -171,7 +171,7 @@ class TestLowerThreshold(BaseOneThreshold):
     def get_side_object():
         return get_two_parts_side()
 
-    def get_algorithm_class(self) -> Type[SegmentationAlgorithm]:
+    def get_algorithm_class(self) -> Type[ROIExtractionAlgorithm]:
         return sa.LowerThresholdAlgorithm
 
 
@@ -193,7 +193,7 @@ class TestUpperThreshold(BaseOneThreshold):
     def get_side_object():
         return get_two_parts_side_reversed()
 
-    def get_algorithm_class(self) -> Type[SegmentationAlgorithm]:
+    def get_algorithm_class(self) -> Type[ROIExtractionAlgorithm]:
         return sa.UpperThresholdAlgorithm
 
 
@@ -259,7 +259,7 @@ class TestRangeThresholdAlgorithm:
 
 
 class BaseFlowThreshold(BaseThreshold, ABC):  # pylint: disable=W0223
-    @pytest.mark.parametrize("sprawl_algorithm_name", sprawl_dict.keys())
+    @pytest.mark.parametrize("sprawl_algorithm_name", flow_dict.keys())
     @pytest.mark.parametrize("compare_op", [operator.eq, operator.ge])
     @pytest.mark.parametrize("components", [2] + list(range(3, 15, 2)))
     def test_multiple(self, sprawl_algorithm_name, compare_op, components):
@@ -267,7 +267,7 @@ class BaseFlowThreshold(BaseThreshold, ABC):  # pylint: disable=W0223
         parameters = self.get_parameters()
         image = self.get_multiple_part(components)
         alg.set_image(image)
-        sprawl_algorithm = sprawl_dict[sprawl_algorithm_name]
+        sprawl_algorithm = flow_dict[sprawl_algorithm_name]
         parameters["sprawl_type"] = {"name": sprawl_algorithm_name, "values": sprawl_algorithm.get_default_values()}
         if compare_op(1, 0):
             parameters["threshold"]["values"]["base_threshold"]["values"]["threshold"] += self.get_shift()
@@ -275,14 +275,14 @@ class BaseFlowThreshold(BaseThreshold, ABC):  # pylint: disable=W0223
         result = alg.calculation_run(empty)
         self.check_result(result, [4000] * components, compare_op, parameters)
 
-    @pytest.mark.parametrize("algorithm_name", sprawl_dict.keys())
+    @pytest.mark.parametrize("algorithm_name", flow_dict.keys())
     def test_side_connection(self, algorithm_name):
         image = self.get_side_object()
         alg = self.get_algorithm_class()()
         parameters = self.get_parameters()
         parameters["side_connection"] = True
         alg.set_image(image)
-        val = sprawl_dict[algorithm_name]
+        val = flow_dict[algorithm_name]
         parameters["sprawl_type"] = {"name": algorithm_name, "values": val.get_default_values()}
         alg.set_parameters(**parameters)
         result = alg.calculation_run(empty)
@@ -312,7 +312,7 @@ class TestLowerThresholdFlow(BaseFlowThreshold):
     get_side_object = staticmethod(get_two_parts_side)
     get_multiple_part = staticmethod(get_multiple_part)
 
-    def get_algorithm_class(self) -> Type[SegmentationAlgorithm]:
+    def get_algorithm_class(self) -> Type[ROIExtractionAlgorithm]:
         return sa.LowerThresholdFlowAlgorithm
 
 
@@ -336,7 +336,7 @@ class TestUpperThresholdFlow(BaseFlowThreshold):
     get_side_object = staticmethod(get_two_parts_side_reversed)
     get_multiple_part = staticmethod(get_multiple_part_reversed)
 
-    def get_algorithm_class(self) -> Type[SegmentationAlgorithm]:
+    def get_algorithm_class(self) -> Type[ROIExtractionAlgorithm]:
         return sa.UpperThresholdFlowAlgorithm
 
 
@@ -786,11 +786,15 @@ class TestSegmentationInfo:
 
 
 def test_bound_info():
-    bi = BoundInfo(lower=np.array([1, 1, 1]), upper=np.array([5, 5, 5]))
-    assert np.all(bi.box_size() == 5)
+    bi = BoundInfo(lower=np.array([1, 1, 1]), upper=np.array([5, 6, 7]))
+    assert np.all(bi.box_size() == [5, 6, 7])
     assert len(bi.box_size()) == 3
     assert len(bi.get_slices()) == 3
-    assert np.all([x == slice(1, 6) for x in bi.get_slices()])
+    assert np.all([x == slice(1, 6 + i) for i, x in enumerate(bi.get_slices())])
+    bi2 = bi.del_dim(1)
+    assert len(bi2.box_size()) == 2
+    assert len(bi2.get_slices()) == 2
+    assert np.all(bi2.box_size() == [5, 7])
 
 
 def test_dict_repr(monkeypatch):
